Package: rgoogleads
Title: Loading Data from 'Google Ads API'
Version: 0.14.1
Authors@R: 
    c(person(given  = "Alexey",
             family = "Seleznev",
             role   = c("aut", "cre"),
             email  = "selesnow@gmail.com",
             comment = c(ORCID = "0000-0003-0410-7385")),
      person(given = "Netpeak",
             role = c("cph")))
Description: Interface for loading data from 'Google Ads API', 
    see <https://developers.google.com/google-ads/api/docs/start>. 
    Package provide function for authorization and loading reports.
License: MIT + file LICENSE
BugReports: https://github.com/selesnow/rgoogleads/issues
URL: https://selesnow.github.io/rgoogleads/,
        https://selesnow.github.io/rgoogleads/docs/,
        https://github.com/selesnow/rgoogleads
Encoding: UTF-8
Imports: gargle (>= 1.2.0), httr, stringr, rlang, dplyr (>= 1.0.0),
        tidyr (>= 1.0.0), jsonlite, snakecase, cli (>= 3.0.0), pbapply,
        purrr, withr, rlist, rvest (>= 1.0.0), memoise, cachem,
        rappdirs, utils, lifecycle
RoxygenNote: 7.3.2
Suggests: rmarkdown, knitr, DT
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-16 14:04:39 UTC; User
Author: Alexey Seleznev [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0410-7385>),
  Netpeak [cph]
Maintainer: Alexey Seleznev <selesnow@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-16 14:50:02 UTC
Built: R 4.5.2; ; 2026-04-17 09:35:09 UTC; unix

Package: rgoogleads
Title: Loading Data from 'Google Ads API'
Version: 0.14.1
Authors@R: 
    c(person(given  = "Alexey",
             family = "Seleznev",
             role   = c("aut", "cre"),
             email  = "selesnow@gmail.com",
             comment = c(ORCID = "0000-0003-0410-7385")),
      person(given = "Netpeak",
             role = c("cph")))
Description: Interface for loading data from 'Google Ads API', 
    see <https://developers.google.com/google-ads/api/docs/start>. 
    Package provide function for authorization and loading reports.
License: MIT + file LICENSE
BugReports: https://github.com/selesnow/rgoogleads/issues
URL: https://selesnow.github.io/rgoogleads/,
        https://selesnow.github.io/rgoogleads/docs/,
        https://github.com/selesnow/rgoogleads
Encoding: UTF-8
Imports: gargle (>= 1.2.0), httr, stringr, rlang, dplyr (>= 1.0.0),
        tidyr (>= 1.0.0), jsonlite, snakecase, cli (>= 3.0.0), pbapply,
        purrr, withr, rlist, rvest (>= 1.0.0), memoise, cachem,
        rappdirs, utils, lifecycle
RoxygenNote: 7.3.2
Suggests: rmarkdown, knitr, DT
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-16 14:04:39 UTC; User
Author: Alexey Seleznev [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0410-7385>),
  Netpeak [cph]
Maintainer: Alexey Seleznev <selesnow@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-16 14:50:02 UTC
Built: R 4.6.0; ; 2026-04-26 18:27:56 UTC; unix

# export
export(gads_auth)
export(gads_deauth)
export(gads_token)
export(gads_has_token)
export(gads_auth_configure)
export(gads_api_key)
export(gads_oauth_app)
export(gads_auth_cache_path)
export(gads_open_auth_cache_folder)
export(gads_developer_token)
export(gads_user)
export(gads_set_login_customer_id)
export(gads_set_customer_id)
export(gads_get_accessible_customers)
export(gads_customer)
export(gads_get_account_hierarchy)
export(gads_get_report)
export(gads_get_campaigns)
export(gads_get_ad_groups)
export(gads_get_ads)
export(gads_get_ad_group_criterions)
export(gads_get_keywords)
export(gads_get_fields)
export(gads_get_metadata)
export(gads_keyword_plan_historical_metrics)
export(gads_keyword_plan_forecast_timeseries)
export(gads_keyword_plan_forecast_metrics)
export(gads_get_geo_targets)
export(gads_last_request_ids)

# import
import(dplyr)
import(tidyr)
import(stringr)
import(purrr)
import(gargle)
import(snakecase)
import(memoise)
import(cachem)
import(rappdirs)
import(utils)

# import from
importFrom(rlang,"%||%", 'caller_env')
importFrom(httr, GET, POST, content, add_headers, headers)
importFrom(jsonlite, toJSON, fromJSON)
importFrom(cli, cli_alert_info, cli_alert_success, cli_alert_danger, cli_alert_warning, cli_div, cli_abort)
importFrom(pbapply, pblapply)
importFrom(rlist, list.filter)
importFrom(rvest, read_html, html_element, html_attr)
importFrom(utils, read.csv)

# rgoogleads: R package for work with Google Ads API

<!-- badges: start -->
[![](https://cranlogs.r-pkg.org/badges/grand-total/rgoogleads)](https://cran.r-project.org/package=rgoogleads)
[![](https://cranlogs.r-pkg.org/badges/rgoogleads?color=lightgrey)](https://cran.r-project.org/package=rgoogleads)
[![](https://cranlogs.r-pkg.org/badges/last-week/rgoogleads?color=lightgrey)](https://cran.r-project.org/package=rgoogleads)
[![CRAN
status](https://www.r-pkg.org/badges/version-ago/rgoogleads)](https://CRAN.R-project.org/package=rgoogleads)
[![R-CMD-check](https://github.com/selesnow/rgoogleads/workflows/R-CMD-check/badge.svg)](https://github.com/selesnow/rgoogleads/actions)
[![Lifecycle: experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
<!-- badges: end -->


## Table of Contents

+ [Links](#links)
+ [Install](#install)
+ [Attach rgoogleads](#attach-rgoogleads)
+ [Main goal and capabilities of rgoogleads](#main-goal-and-capabilities-of-rgoogleads)
+ [Privacy Policy](#privacy-policy)
    + [Authorization process](#authorization-process)
    + [Key points](#key-points)
    + [Use own OAuth client](#use-own-oauth-client)
+ [Obtain Own Developer Token and create own OAuth client](#obtain-own-developer-token-and-create-own-oauth-client)
+ [Example of use rgoogleads](#example-of-use-rgoogleads)
+ [Package help](#package-help)
+ [Package chage log, news and updates](#package-chage-log-news-and-updates)
+ [Bug report and support](#bug-report-and-support)
+ [Tutorials](#tutorials)
+ [Author](#author)

## Links

This section contains links to the package documentation, as well as other useful links that may be useful to the package users.

* [rgoogleads package documentation site](https://selesnow.github.io/rgoogleads/docs/)
* [CRAN page](https://cran.r-project.org/package=rgoogleads)
* [Google Ads API documentation](https://developers.google.com/google-ads/api/docs/start)
* [Официальный плейлист с видео уроками по работе с rgoogleads](https://www.youtube.com/playlist?list=PLD2LDq8edf4qprTxRcflDwV9IvStiChHi)

## Install

You can install `rgoogleads` from [CRAN](https://cran.r-project.org/package=rgoogleads) or [GitHub](https://github.com):

```r
# install from CRAN
install.packages('rgoogleads')
```

```r
# install from github
devtools::install_github('selesnow/rgoogleads')
```

## Attach rgoogleads

```r
library(rgoogleads)
```

## Main goal and capabilities of rgoogleads

`rgoogleads` is R client for work with Google Ads API v8. Main goal of `rgoogleads` - load report data from Google Ads client account into R for analysis and visualizations.

Capabilities of `rgoogleads`:

* Authorization in the Google Ads API
* Loading a list of top-level accounts
* Loading the entire hierarchy of accounts from manager accounts
* Loading list of Google Ads client account objects: campaigns, ad groups, ads, etc.
* Loading statistics from Google Ads client account
* Loading resource metadata, resource fields, segments and metrics
* Loading forecast and historical metrics from Keyword Planning.

## Privacy Policy

The `rgoogleads` package for authorization uses the [gargle](https://gargle.r-lib.org/) package, the credentials obtained during authorization are stored exclusively on your local PC, you can find out the folder into which the credentials are cached using the `gads_auth_cache_path()` function.

For loading data from Google Ads Account `rgoogleads` needs `https://www.googleapis.com/auth/adwords` scope (Manage Your Google AdWords Campaigns), see [official Google Ads API documentation](https://developers.google.com/google-ads/api/docs/oauth/internals#scope). 

The package does not transfer your credentials or data obtained from your advertising accounts to third parties, however, the responsibility for information leakage remains on the side of the package user. The author does not bear any responsibility for their safety, be careful when transferring cached credentials to third parties.

For more details, I recommend that you read the following articles from the official documentation of the gargle package:

* [Stewarding the cache of user tokens](https://tidyverse.org/blog/2021/07/gargle-1-2-0/)
* [Auth when using R in the browser](https://cran.r-project.org/package=gargle/vignettes/auth-from-web.html)
* [How gargle gets tokens](https://cran.r-project.org/package=gargle/vignettes/how-gargle-gets-tokens.html)

### Authorization process

You run `gads_auth('me@gmail.com')` and start [OAuth Dance](https://medium.com/typeforms-engineering-blog/the-beginners-guide-to-oauth-dancing-4b8f3666de10) in the browser:

![Typical OAuth dance in the browser, when initiated from within R](http://img.netpeak.ua/alsey/1OE9JZ2.png)

Upon success, you see this message in the browser:

`Authentication complete. Please close this page and return to R.`

And you credentials cached locally on your PC in the form of RDS files.

### Key points
* By default, gargle caches user tokens centrally, at the user level, and their keys or labels also convey which Google identity is associated with each token.
* Token storage relies on serialized R objects. That is, tokens are stored locally on your PC in the form of RDS files.

### Use own OAuth client
You can use own OAuth app:

```r
app <- httr::oauth_app(appname = "app name", key = "app id", secret = "app secret")
gads_auth_configure(app = app)

# or from json file 
gads_auth_configure(path = 'D:/ga_auth/app.json')

# run authorization
gads_auth('me@gmail.com')
```

## Obtain Own Developer Token and create own OAuth client

For obtain own developer token and OAuth client read the following documentation:

* [Obtain Your Developer Token](https://developers.google.com/google-ads/api/docs/get-started/dev-token)
* [Configure a Google API Console Project for the Google Ads API](https://developers.google.com/google-ads/api/docs/get-started/oauth-cloud-project)

## Example of use rgoogleads

```r
library(rgoogleads)

# set own oauth app
gads_auth_configure(path = 'C:/auth/app.json')
# set your developer token if needed, or use default developer token
gads_auth(email = 'me@gmail.com', developer_token = "own developer token")

# get list of accessible accounts
my_accounts <- gads_get_accessible_customers()

# set manager account id (options)
gads_set_login_customer_id('xxx-xxx-xxxx')

# set client account id
gads_set_customer_id('xxx-xxx-xxxx')

# load report data
ad_group_report <- gads_get_report(
  resource    = "ad_group",
  fields      = c("ad_group.campaign",
                  "ad_group.id",
                  "ad_group.name",
                  "ad_group.status",
                  "metrics.clicks",
                  "metrics.cost_micros"),
  date_from   = "2021-06-10",
  date_to     = "2021-06-17",
  where       = "ad_group.status = 'ENABLED'",
  order_by    = c("metrics.clicks DESC", "metrics.cost_micros")
)
```

## Package help
For get help of `rgoogleads` use `?rgoogleads` or `?gads_get_report`.

## Package chage log, news and updates
You can follow the package updates at the [link](https://github.com/selesnow/rgoogleads/blob/master/NEWS.md).

## Bug report and support
If you encounter an error in the package, or you have suggestions for improving its functionality, you can create a issue using the [link](https://github.com/selesnow/rgoogleads/issues).

## Tutorials

This section contains links to articles dedicated to `rgoogleads`.

* Русскоязычные материалы
    * [Официальный плейлист с видео уроками](https://www.youtube.com/playlist?list=PLD2LDq8edf4qprTxRcflDwV9IvStiChHi)
        * [Авторизация в Google Ads API и запрос иерархии аккаунтов](https://www.youtube.com/watch?v=nYak5sVj07k&list=PLD2LDq8edf4qprTxRcflDwV9IvStiChHi&index=1), *Alexey Seleznev*
        * [Запрос отчётов из Google Ads API](https://www.youtube.com/watch?v=np67MNj2-JU), *Alexey Seleznev*
    * [Миграция с Google AdWords API на Google Ads API: подробный мануал](https://netpeak.net/ru/blog/migratsiya-s-google-adwords-api-na-google-ads-api-podrobnyy-manual/), *Alexey Seleznev*
    * [Как визуализировать показатель качества ключевых слов — рецепт скрипта на языке R](https://netpeak.net/ru/blog/kak-vizualizirovat-pokazatel-kachestva-klyuchevyh-slov-retsept-skripta-na-yazyke-r/), *Alexey Seleznev*
    * [Как оценить потерянный доход в Google Ads с помощью языка R](https://netpeak.net/ru/blog/kak-otsenit-poteryannyi-dokhod-v-google-adwords-s-pomoshch-yu-yazyka-r/), *Alexey Seleznev*
    * [Доклад: Зачем интернет маркетологу понимать что такое API. Разбираем устройство API Google Ads (Конференция 8P 2021)](https://youtu.be/wtXVwOBo518), *Alexey Seleznev*
* English
    * [Migrating from AdWords API to Google Ads API. A Comprehensive Guideline](https://netpeak.us/blog/migrating-from-adwords-api-to-google-ads-api-a-comprehensive-guideline/), *Alexey Seleznev*

If you have published an article about `rgoogleads` let me know about it in [telegram](https://t.me/AlexeySeleznev) or [email](mailto:selesnow@gmail.com) and I will add it to the list of links.

## Author
Alexey Seleznev, Head of analytics dept. at [Netpeak](https://netpeak.us/)
<Br>Telegram Channel: [R4marketing](https://t.me/R4marketing)
<Br>email: selesnow@gmail.com
<Br>facebook: [facebook.com/selesnow](https://www.facebook.com/selesnow)
<Br>blog: [alexeyseleznev.wordpress.com](https://alexeyseleznev.wordpress.com/)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  eval = FALSE,
  comment = "#>"
)

## ----eval = TRUE, results = "asis", echo=FALSE--------------------------------

migrate_table <- data.frame(
  operation = c("Авторизация", "Запрос метаданных", "Запрос отчётов"),
  radwords = c("doAuth()", "reports(), metrics()", "statement() + getData()"),
  rgoogleads = c("gads_auth_configure() + gads_auth()", "gads_get_metadata(), gads_get_fields()", "gads_get_report()")
  )

DT::datatable(
  migrate_table, 
  colnames = c("Операция", "RAdwords", "rgoogleads"),
  options = list(pageLength = 5, dom = 'tip'))



## ----eval = TRUE, results = "asis", echo=FALSE--------------------------------
library(dplyr)
library(rvest)
reports_table <- data.frame(
  adwords = c("ACCOUNT_PERFORMANCE_REPORT", "AD_PERFORMANCE_REPORT", "ADGROUP_PERFORMANCE_REPORT", "AGE_RANGE_PERFORMANCE_REPORT",
              "AUDIENCE_PERFORMANCE_REPORT", "AUTOMATIC_PLACEMENTS_PERFORMANCE_REPORT", "BID_GOAL_PERFORMANCE_REPORT", "BUDGET_PERFORMANCE_REPORT",
              "CALL_METRICS_CALL_DETAILS_REPORT", "CAMPAIGN_AD_SCHEDULE_TARGET_REPORT", "CAMPAIGN_CRITERIA_REPORT", "CAMPAIGN_PERFORMANCE_REPORT",
              "CAMPAIGN_SHARED_SET_REPORT", "CAMPAIGN_LOCATION_TARGET_REPORT", "CLICK_PERFORMANCE_REPORT", "DISPLAY_KEYWORD_PERFORMANCE_REPORT"),
  ads     = c("customer", "ad_group_ad", "ad_group", "age_range_view",
              "campaign_audience_view, ad_group_audience_view", "group_placement_view", "bidding_strategy", "campaign_budget",
              "call_view", "ad_schedule_view", "campaign_criterion", "campaign",
              "campaign_shared_set", "location_view", "click_view", "display_keyword_view"))
  
# reports <- read_html("https://developers.google.com/google-ads/api/docs/migration/mapping") %>% 
#            html_element(css = ".responsive") %>% 
#            html_table(header = TRUE)

DT::datatable(
  reports_table, 
  colnames = c("Тип отчёта в Google AdWords API", "Ресурс в Google Ads API"),
  options = list(pageLength = 20)
  )


## -----------------------------------------------------------------------------
# library(RAdwords)
# 
# # авторизация
# adwords_auth <- doAuth()
# 
# # составляем запрос
# query <- statement(
#   select = c('CampaignName',
#             'Date',
#             'Clicks'),
#   report = 'CAMPAIGN_PERFORMANCE_REPORT',
#   start  = '2021-06-01',
#   end    = '2021-06-30'
# )
# 
# # загрузка данных
# data1 <- getData(
#   clientCustomerId = 'xxx-xxx-xxxx',
#   statement        = query,
#   google_auth      = adwords_auth
# )
# 

## -----------------------------------------------------------------------------
# library(rgoogleads)
# 
# # авторизация
# gads_auth_configure(path = 'D:/ga_auth/app.json')
# gads_auth(email = 'me@gmail.com')
# 
# # загрузка данных
# data2 <- gads_get_report(
#   resource = 'campaign',
#   fields   = c('campaign.name',
#               'segments.date',
#               'metrics.clicks'),
#   date_from         = '2021-06-01',
#   date_to           = '2021-06-30',
#   customer_id       = 'xxx-xxx-xxxx',
#   login_customer_id = 'xxx-xxx-xxxx'
# )

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  eval = FALSE,
  comment = "#>"
)

## ----eval = TRUE, results = "asis", echo=FALSE--------------------------------

migrate_table <- data.frame(
  operation = c("Authorization", "Metadata request", "Report request"),
  radwords = c("doAuth()", "reports(), metrics()", "statement() + getData()"),
  rgoogleads = c("gads_auth_configure() + gads_auth()", "gads_get_metadata(), gads_get_fields()", "gads_get_report()")
  )

DT::datatable(
  migrate_table, 
  colnames = c("Operation", "RAdwords", "rgoogleads"),
  options = list(pageLength = 5, dom = 'tip'))



## ----eval = TRUE, results = "asis", echo=FALSE--------------------------------
library(dplyr)
library(rvest)
reports_table <- data.frame(
  adwords = c("ACCOUNT_PERFORMANCE_REPORT", "AD_PERFORMANCE_REPORT", "ADGROUP_PERFORMANCE_REPORT", "AGE_RANGE_PERFORMANCE_REPORT",
              "AUDIENCE_PERFORMANCE_REPORT", "AUTOMATIC_PLACEMENTS_PERFORMANCE_REPORT", "BID_GOAL_PERFORMANCE_REPORT", "BUDGET_PERFORMANCE_REPORT",
              "CALL_METRICS_CALL_DETAILS_REPORT", "CAMPAIGN_AD_SCHEDULE_TARGET_REPORT", "CAMPAIGN_CRITERIA_REPORT", "CAMPAIGN_PERFORMANCE_REPORT",
              "CAMPAIGN_SHARED_SET_REPORT", "CAMPAIGN_LOCATION_TARGET_REPORT", "CLICK_PERFORMANCE_REPORT", "DISPLAY_KEYWORD_PERFORMANCE_REPORT"),
  ads     = c("customer", "ad_group_ad", "ad_group", "age_range_view",
              "campaign_audience_view, ad_group_audience_view", "group_placement_view", "bidding_strategy", "campaign_budget",
              "call_view", "ad_schedule_view", "campaign_criterion", "campaign",
              "campaign_shared_set", "location_view", "click_view", "display_keyword_view"))
  
# reports <- read_html("https://developers.google.com/google-ads/api/docs/migration/mapping") %>% 
#            html_element(css = ".responsive") %>% 
#            html_table(header = TRUE)

DT::datatable(
  reports_table, 
  colnames = c("Тип отчёта в Google AdWords API", "Ресурс в Google Ads API"),
  options = list(pageLength = 20)
  )


## -----------------------------------------------------------------------------
# library(RAdwords)
# 
# # auth
# adwords_auth <- doAuth()
# 
# # create request
# query <- statement(
#   select = c('CampaignName',
#             'Date',
#             'Clicks'),
#   report = 'CAMPAIGN_PERFORMANCE_REPORT',
#   start  = '2021-06-01',
#   end    = '2021-06-30'
# )
# 
# # data import
# data1 <- getData(
#   clientCustomerId = 'xxx-xxx-xxxx',
#   statement        = query,
#   google_auth      = adwords_auth
# )
# 

## -----------------------------------------------------------------------------
# library(rgoogleads)
# 
# # auth
# gads_auth_configure(path = 'D:/ga_auth/app.json')
# gads_auth(email = 'me@gmail.com')
# 
# # data import
# data2 <- gads_get_report(
#   resource = 'campaign',
#   fields   = c('campaign.name',
#               'segments.date',
#               'metrics.clicks'),
#   date_from         = '2021-06-01',
#   date_to           = '2021-06-30',
#   customer_id       = 'xxx-xxx-xxxx',
#   login_customer_id = 'xxx-xxx-xxxx'
# )

# cache
gads_get_fields_cached <- memoise(gads_get_fields)

# make gaul query
gads_make_query <- function(
  resource              = 'campaign',
  fields                = c('campaign.id',
                            'campaign.name',
                            'customer.id',
                            'customer.descriptive_name',
                            'campaign.status',
                            'segments.date',
                            'metrics.all_conversions',
                            'metrics.clicks',
                            'metrics.cost_micros',
                            'metrics.ctr',
                            'metrics.impressions',
                            'metrics.interaction_rate',
                            'metrics.interactions',
                            'metrics.invalid_clicks'),
  where                 = NULL,
  order_by              = NULL,
  limit                 = NULL,
  parameters            = NULL,
  date_from             = Sys.Date() - 15,
  date_to               = Sys.Date() - 1,
  during                = c(NA, "TODAY", "YESTERDAY", "LAST_7_DAYS", "LAST_BUSINESS_WEEK", "THIS_MONTH", "LAST_MONTH", "LAST_14_DAYS", "LAST_30_DAYS", "THIS_WEEK_SUN_TODAY", "THIS_WEEK_MON_TODAY", "LAST_WEEK_SUN_SAT", "LAST_WEEK_MON_SUN")
) {

  # check args
  during <- match.arg(during, choices = c(NA, "TODAY", "YESTERDAY", "LAST_7_DAYS", "LAST_BUSINESS_WEEK", "THIS_MONTH", "LAST_MONTH", "LAST_14_DAYS", "LAST_30_DAYS", "THIS_WEEK_SUN_TODAY", "THIS_WEEK_MON_TODAY", "LAST_WEEK_SUN_SAT", "LAST_WEEK_MON_SUN"))

  # --------------
  # compose query
  # select block
  fields <- gsub("[\\s\\n\\t]", "",  fields, perl = TRUE) %>%
    tolower() %>%
    str_c('\t', .,collapse = ', \n')

  # check of set any period fields
  if ( any(is.null(date_from), is.null(date_to)) & is.null(where) ) {
    where_clause <- ""
  } else if ( !is.na(during) ) {
    where <- str_c(where, collapse = " \nAND ")

    if ( where == "" ) {
      where_clause <- str_glue("WHERE segments.date DURING {during}")
    } else {
      where <- str_c(where, collapse = " \nAND ")
      where_clause <- str_glue("WHERE segments.date DURING {during} \nAND {where}")
    }

  } else if ( any(is.null(date_from), is.null(date_to)) ) {
    where <- str_c(where, collapse = " \nAND ")
    where_clause <- str_glue("WHERE {where}")
  } else {
    where <- str_c(where, collapse = " \nAND ")
    sd <- format(as.Date(date_from), '%Y-%m-%d')
    fd <- format(as.Date(date_to), '%Y-%m-%d')
    where_clause <- ifelse( is.null(where), str_glue("WHERE segments.date BETWEEN '{sd}' AND '{fd}'"), str_glue("WHERE segments.date BETWEEN '{sd}' AND '{fd}' \nAND {where}") )
  }

  # fix last AND
  where_clause    <- gsub('(\nAND )$', '', where_clause)
  # params block
  params_clause   <- ifelse( is.null(parameters), '', str_glue('PARAMETERS {parameters}') )
  # order by block
  order_by_clause <- ifelse( is.null(order_by), '', str_glue('ORDER BY {str_c(order_by, collapse=", ")}'))
  # limit block
  limit_clause    <- ifelse( is.null(limit), '', str_glue('LIMIT {limit}') )

  # make query
  gaql_query <- str_glue(
       'SELECT',
       '{fields}',
       'FROM {resource}',
       '{where_clause}',
       '{order_by_clause}',
       '{limit_clause}',
       '{params_clause}',
       .sep = '\n'
       )

  return(gaql_query)

}


# make request
gads_make_request <- function(
    http_method       = "POST",
    body              = NULL,
    api_version       = options("gads.api.version"),
    api_method        = "googleAds:searchStream",
    customer_id       = getOption('gads.customer.id'),
    login_customer_id = getOption('gads.login.customer.id'),
    verbose           = TRUE
  ) {

  # send query
  out <- request_build(
    method   = http_method,
    body     = body,
    path     = str_glue('{api_version}/{api_method}'),
    token    = gads_token(),
    base_url = getOption('gads.base.url')
  )

  # send request
  ans <- request_retry(
    out,
    encode = 'json',
    add_headers(
      `developer-token`= gads_developer_token(),
      `login-customer-id` = login_customer_id
      )
  )

  # --------------
  # get answer
  if (verbose) cli_alert_info('Get answer query')
  out <- response_process(ans, error_message = gads_check_errors2)

  # requests_ids
  if ( !is.null(ans$headers$`request-id`) ) {

    rq_ids <- unique(ans$headers$`request-id`)
    rgoogleads$last_request_id <- rq_ids
    if (verbose) cli_alert_info(c("Your request ids: ", rq_ids))

  } else {

    rgoogleads$last_request_id <- unique(sapply(out, function(x) x$requestId))
    rq_ids <- str_c(rgoogleads$last_request_id, collapse = ', ')
    if (verbose) cli_alert_info(c("Your request ids: ", rq_ids))

  }

  return(out)

}

#' Download CSV of geo targets
#'
#' @param doc_page Link to Google Ads API Reference page
#' @param file_link Link to csv file, default is 'auto'
#'
#' @return data.frame with geo targets dictionary
#' @export
#' @seealso \href{https://developers.google.com/google-ads/api/reference/data/geotargets?hl=en}{Google Ads Geo Targets document page}
#'
#' @examples
#' \dontrun{
#' geo_dict <- gads_get_geo_targets()
#' }
gads_get_geo_targets <- function(
  doc_page  = 'https://developers.google.com/google-ads/api/reference/data/geotargets',
  file_link = 'auto'
  ) {

  # if auto detect file link
  if (file_link == 'auto') {

    file_link <- read_html(doc_page) %>%
                 html_element(xpath = "//a[contains(text(), 'Latest')]") %>%
                 html_attr('href') %>%
                 str_c('https://developers.google.com/', ., collapse = '')

  }

  # download zip
  zip_file <- tempfile(fileext = '.zip')
  download.file(file_link, zip_file)

  # unzip csv file
  file_name <- unzip(zipfile = zip_file, list = T)
  unzip(zipfile = zip_file)

  # read file
  data <- read.csv(file_name$Name) %>%
          rename_with(getOption('gads.column.name.case.fun'))

  # remove temp file
  file.remove(c(zip_file, file_name$Name))

  # success msg
  cli_alert_success('Success! Loaded {nrow(data)} rows!')

  # return
  return(data)

}

#' Helper function for check api answer on error
#'
#' @param out API answer
#' @param client_id Google Ads Customer id
#' @param verbose Console output
#' @param request_id Api request id
#'
#' @return stop the function when api request faild
#'
gads_check_errors <- function(out, client_id = NULL, verbose = FALSE, request_id) {
   return("ERROR!!")
  # check for empty data
  if ( length(out) == 0 ) {
    msg <- 'empty request, please check client_id, date_from, date_to and where arguments, fox and repeate query'
    if (verbose) cli_alert_warning(c(client_id, ": ", msg, ". Request ID: ", request_id))
    return(NULL)
  }

  # check simple answer
  if ( !is.null(out$error) ) {
    msg <- try(out$error$details[[1]]$errors[[1]]$message)
    cli_alert_danger(c(client_id, ": ", msg))
    cli_alert_danger(c("Request ID: ", request_id))
    return(paste(client_id, msg))
  }

  # check multi answer
  err <- try(!is.null(out[[1]]$error), silent = TRUE)

  # if answer is multiple
  if ( ! "try-error" %in% class(err) ) {

    if ( !is.null(out[[1]]$error) ) {
      msg <- ifelse(is.null(out[[1]]$error$details[[1]]$errors[[1]]$message), out[[1]]$error$message, paste(out[[1]]$error$message, out[[1]]$error$details[[1]]$errors[[1]]$message, sep = ": "))
      cli_alert_danger(c(client_id, ": ", msg))
      cli_alert_danger(c("Request ID: ", request_id))
      cli_alert_danger("You can use gads_last_request_ids() for get last request id, if you want send ticket to google ads api support.")
      return(paste(client_id, msg))
     }

  }

  unlist(out)

}



# новая версия ------------------------------------------------------------

gads_check_errors2 <- function(resp, call = caller_env()) {

  # response
  content <- response_as_json(resp, call = call)

  # variables
  client_id  <- gads_customer_id_from_env()
  request_id <- gads_last_request_ids()

  # verbose options
  verbose <- getOption('gads.multi.account.verbos')

  # check for empty data
  if ( length(content) == 0 ) {
    msg <- 'empty request, please check client_id, date_from, date_to and where arguments, fox and repeate query'
    if (verbose) cli_alert_warning(c(client_id, ": ", msg, ". Request ID: ", request_id))
    return(msg)
  }

  # check simple answer
  if ( !is.null(content$error) ) {

    msg <- ifelse(
      is.null(content$error$details[[1]]$errors[[1]]$message),
      content$error$message,
      content$error$details[[1]]$errors[[1]]$message
      )

    if (verbose) cli_alert_danger(c("\n", client_id, ": ", msg))
    if (verbose) cli_alert_danger(c("\n", "Request ID: ", request_id))
    return(paste(client_id, msg))

  }

  # check multi answer
  err <- try(!is.null(content[[1]]$error), silent = TRUE)

  # if answer is multiple
  if ( ! "try-error" %in% class(err) ) {

    if ( !is.null(content[[1]]$error) ) {
      msg <- ifelse(is.null(content[[1]]$error$details[[1]]$errors[[1]]$message), content[[1]]$error$message, paste(content[[1]]$error$message, content[[1]]$error$details[[1]]$errors[[1]]$message, sep = ": "))

      if (verbose) cli_alert_danger(c(client_id, ": ", msg))
      if (verbose) cli_alert_danger(c("\nRequest ID: ", request_id))
      if (verbose) cli_alert_danger("\nYou can use gads_last_request_ids() for get last request id, if you want send ticket to google ads api support.")
      return(paste(client_id, msg))
    }

  }

  return('Unknown error')

}

rgoogleads <- new.env(parent = emptyenv())
rgoogleads$last_request_id <- NULL
rgoogleads$customer_id     <- NULL
invisible(rgoogleads)

#' Get last API request ID for Google Ads API support ticket
#'
#' @return Request ID
#' @export
#'
#' @examples
#' \dontrun{
#' gads_last_request_ids()
#' }
gads_last_request_ids <- function() {
  return(rgoogleads$last_request_id)
}

#' Write customer id for error message
#' @param customer_id Your client customer id
#' @return only set customer id into env
gads_customer_id_to_env <- function(customer_id) {
  customer_id <- str_replace_all(customer_id, "-", "") %>%
                 str_replace('(\\d{3})(\\d{3})(\\d{4})', '\\1-\\2-\\3')

  rgoogleads$customer_id <- customer_id
}

#' Get customer id for error message
#'
#' @return only set customer id into env
#'
gads_customer_id_from_env <- function() {
  return(rgoogleads$customer_id)
}

.onLoad <- function(libname, pkgname) {

  # auth object
  .auth <<- gargle::init_AuthState(
    package     = "rgoogleads",
    auth_active = TRUE
  )

  # where function
  utils::globalVariables("where")

  ## adwords developer token
  if ( Sys.getenv("GADS_DEVELOPER_TOKEN") != "" ) {

    developer_token <- Sys.getenv("GADS_DEVELOPER_TOKEN")
    cli_alert_info('Set developer token from environt variables')

  } else {

    developer_token <- 'EBkkx-znu2cZcEY7e74smg'

  }

  ## login customer id
  if ( Sys.getenv("GADS_LOGIN_CUSTOMER_ID") != "" ) {

    login_customer_id <- Sys.getenv("GADS_LOGIN_CUSTOMER_ID")
    cli_alert_info('Set login customer id from environt variables')

  } else {

    login_customer_id <- NULL

  }

  ## login customer id
  if ( Sys.getenv("GADS_CUSTOMER_ID") != "" ) {

    customer_id <- Sys.getenv("GADS_CUSTOMER_ID")
    cli_alert_info('Set customer id from environt variables')

  } else {

    customer_id <- NULL

  }

  if ( Sys.getenv("GADS_EMAIL") != "" ) {

    gads_email <- Sys.getenv("GADS_EMAIL")
    cli_alert_info('Set email from environt variables')

  } else {

    gads_email <- NULL

  }

  # options
  op <- options()
  op.gads <- list(gads.developer.token      = developer_token,
                  gads.api.version          = "v23",
                  gads.login.customer.id    = login_customer_id,
                  gads.customer.id          = customer_id,
                  gads.multi.account.verbos = FALSE,
                  gads.base.url             = 'https://googleads.googleapis.com/',
                  gads.column.name.case.fun = snakecase::to_snake_case,
                  gargle_oauth_email        = gads_email,
                  gads.show_gaql_query      = FALSE)

  toset <- !(names(op.gads) %in% names(op))
  if (any(toset)) options(op.gads[toset])

  invisible()
}

.onAttach <- function(lib, pkg,...){

  packageStartupMessage(rgoogleadsWelcomeMessage())

}


rgoogleadsWelcomeMessage <- function(){
  # library(utils)

  paste0("\n",
         "---------------------\n",
         "Welcome to rgoogleads version ", utils::packageDescription("rgoogleads")$Version, "\n",
         "\n",
         "Author:           Alexey Seleznev (Head of analytics dept at Netpeak).\n",
         "Telegram channel: https://t.me/R4marketing \n",
         "YouTube channel:  https://www.youtube.com/R4marketing/?sub_confirmation=1 \n",
         "Email:            selesnow@gmail.com\n",
         "Site:             https://selesnow.github.io \n",
         "Blog:             https://alexeyseleznev.wordpress.com \n",
         "Facebook:         https://facebook.com/selesnown \n",
         "Linkedin:         https://www.linkedin.com/in/selesnow \n",
         "\n",
         "Using Googla Ads API version: ", getOption('gads.api.version'), "\n",
         "\n",
         "Type ?rgoogleads for the main documentation.\n",
         "The github page is: https://github.com/selesnow/rgoogleads/\n",
         "Package site: https://selesnow.github.io/rgoogleads/docs\n",
         "Package lessons playlist: https://www.youtube.com/playlist?list=PLD2LDq8edf4qprTxRcflDwV9IvStiChHi\n",
         "\n",
         "Suggestions and bug-reports can be submitted at: https://github.com/selesnow/rgoogleads/issues\n",
         "Or contact: <selesnow@gmail.com>\n",
         "\n",
         "\tTo suppress this message use:  ", "suppressPackageStartupMessages(library(rgoogleads))\n",
         "---------------------\n"
  )
}

#' Get Ad Groups Dictionary From Google Ads Client Account
#'
#' @inheritParams gads_get_report
#'
#' @param fields character vector, list of report fields, all report has own fields list, for example \href{https://developers.google.com/google-ads/api/fields/v23/ad_group}{see field list of ad group report}.
#'
#' @seealso \href{https://developers.google.com/google-ads/api/fields/v23/ad_group_query_builder}{Google Ads Query Builder}
#'
#' @return tibble with ad group dicrionary
#' @export
#' @examples
#' \dontrun{
#' # set client customer id
#' gads_set_login_customer_id('xxx-xxx-xxxx')
#'
#' # set manager id if you work under MCC
#' gads_set_customer_id('xxx-xxx-xxxx')
#'
#' # load ad groups list
#' adgroups <- gads_get_ad_groups(
#'     where = 'ad_group.status = "ENABLED"'
#' )
#'
#' }
gads_get_ad_groups <- function(
  customer_id           = getOption('gads.customer.id'),
  fields                = c('ad_group.id',
                            'ad_group.name',
                            'ad_group.status',
                            'ad_group.ad_rotation_mode',
                            'ad_group.base_ad_group',
                            'ad_group.campaign',
                            'campaign.id',
                            'ad_group.display_custom_bid_dimension',
                            'ad_group.effective_target_cpa_source',
                            'ad_group.effective_target_roas',
                            'ad_group.effective_target_roas_source',
                            'ad_group.final_url_suffix',
                            'ad_group.target_roas',
                            'ad_group.type',
                            'ad_group.url_custom_parameters',
                            'ad_group.tracking_url_template',
                            'customer.id',
                            'customer.descriptive_name'),
  where                 = NULL,
  order_by              = NULL,
  limit                 = NULL,
  parameters            = NULL,
  login_customer_id     = getOption('gads.login.customer.id'),
  include_resource_name = FALSE,
  cl                    = NULL,
  verbose               = TRUE
) {

  # user gads_get_report
  res <- gads_get_report(
    resource          = 'ad_group',
    fields            = fields,
    where             = where,
    order_by          = order_by,
    limit             = limit,
    parameters        = parameters,
    customer_id       = customer_id,
    login_customer_id = login_customer_id,
    verbose           = verbose,
    cl                = cl,
    date_from         = NULL,
    date_to           = NULL
  )

  # renaming to snale case
  res <- rename_with(res, function(x) str_remove(str_to_lower(x), 'ad\\_?group\\_?'), matches('ad\\_?group', ignore.case = TRUE) ) %>%
    rename_with(getOption('gads.column.name.case.fun'))

  # fix date
  if ( any(str_detect( str_to_lower(names(res)), 'date' )) ) {
    res <- mutate(res,
                  across(matches('date', ignore.case = TRUE), as.Date))
  }

  # return
  return(res)

}

#' Get metada of object, RESOURCE, ATTRIBUTE, METRIC or SEGMENT
#'
#' @param category Object category
#' @param fields Metadata fields
#'
#' @return tibble with object metadata
#' important arrays in result:
#' \describe{
#'   \item{attributeResources}{Resources that can be using in \code{resource} argument in \code{\link{gads_get_report}}.}
#'   \item{metrics}{Metrics that are available to be selected with the resource in the \code{field} argument in \code{\link{gads_get_report}}. Only populated for fields where the category is RESOURCE.}
#'   \item{segments}{Segment keys that can be selected with the resource in the \code{field} argument in \code{\link{gads_get_report}}. These segment the metrics specified in the query. Only populated for fields where the category is RESOURCE.}
#'   \item{selectableWith}{Fields that can be selected alongside a given field, when not in the FROM clause. This attribute is only relevant when identifying resources or segments that are able to be selected in a query where they are not included by the resource in the FROM clause. As an example, if we are selecting \code{ad_group.id} and \code{segments.date} from \code{ad_group}, and we want to include attributes from \code{campaign}, we would need to check that \code{segments.date} is in the selectableWith attribute for campaign, since it's being selected alongside the existing \code{segments.date} field.}
#' }
#' @export
#'
#' @seealso \href{https://ads-developers.googleblog.com/2021/04/the-query-builder-blog-series-part-3.html}{The Query Builder Blog Series: Part 3 - Creating a Resource Schema}
#' and \href{https://developers.google.com/google-ads/api/docs/concepts/field-service}{Resource Metadata API documentation}
#'
#' @examples
#' \dontrun{
#' # get resource list
#' resources <- gads_get_metadata("RESOURCE")
#'
#' # get list of all objects
#' metadata <- gads_get_metadata("ALL")
#'
#' }
gads_get_metadata <- function(
  category = c("RESOURCE", "ATTRIBUTE", "METRIC", "SEGMENT", 'ALL'),
  fields   = c("name",
               "category",
               "data_type",
               "selectable",
               "filterable",
               "sortable",
               "selectable_with",
               "metrics",
               "segments",
               "is_repeated",
               "type_url",
               "enum_values",
               "attribute_resources")
) {

  # check category
  category <- toupper(category)
  match.arg(category)

  # --------------
  # compose query
  # select block
  fields <- gsub("[\\s\\n\\t]", "", tolower(fields), perl = TRUE) %>%
            tolower() %>%
            str_c(collapse = ', ')

  # where block
  if ( category == "ALL" ) {
    where_clause <- ""
  } else {
    where_clause <- str_glue("WHERE category = '{category}'")
  }

  # --------------
  # build GAQL Query
  body <- list(query =
                 str_glue('
       SELECT {fields}

       {where_clause}')) %>%
    toJSON(auto_unbox = T, pretty = T)

  # --------------
  # build query
  out <- request_build(
    method   = "POST",
    path     = str_glue('{options("gads.api.version")}/googleAdsFields:search'),
    body     = body,
    token    = gads_token(),
    base_url = getOption('gads.base.url')
  )

  # send request
  ans <- request_retry(
    out,
    encode = "json",
    add_headers(`developer-token`= gads_developer_token())
  )

  # request id
  rq_ids <- headers(ans)$`request-id`
  rgoogleads$last_request_id <- rq_ids

  # pars result
  data <- response_process(ans, error_message = gads_check_errors2)

  # parse
  res <- tibble(data = data$results) %>%
         unnest_wider(data) %>%
         rowwise() %>%
         mutate( across( where(is.list), function(col) if_else(is.null(col), list(col), list(unlist(col))) ) ) %>%
         rename_with( getOption('gads.column.name.case.fun') )

  # success msg
  cli_alert_success('Success! Loaded {nrow(res)} rows!')

  # return
  return(res)

}

#' Get Campaigns Dictionary From Google Ads Client Account
#'
#' @inheritParams gads_get_report
#'
#' @param fields character vector, list of report fields, all report has own fields list, for example \href{https://developers.google.com/google-ads/api/fields/v23/campaign}{see field list of campaigns report}.
#'
#' @seealso \href{https://developers.google.com/google-ads/api/fields/v23/campaign_query_builder}{Google Ads Query Builder}
#'
#' @return tibble with campaings dicrionary
#' @export
#' @examples
#' \dontrun{
#' # set client customer id
#' gads_set_login_customer_id('xxx-xxx-xxxx')
#'
#' # set manager id if you work under MCC
#' gads_set_customer_id('xxx-xxx-xxxx')
#'
#' # load campaing list
#' camps <- gads_get_campaigns(
#'     where = "campaign.status = 'ENABLED'"
#' )
#'
#' }
gads_get_campaigns <- function(
  fields                = c('campaign.id',
                            'campaign.name',
                            'campaign.accessible_bidding_strategy',
                            'campaign.ad_serving_optimization_status',
                            'campaign.advertising_channel_sub_type',
                            'campaign.advertising_channel_type',
                            'campaign.app_campaign_setting.app_id',
                            'campaign.app_campaign_setting.app_store',
                            'campaign.base_campaign',
                            'campaign.bidding_strategy',
                            'campaign.app_campaign_setting.bidding_strategy_goal_type',
                            'campaign.campaign_budget',
                            'campaign.bidding_strategy_type',
                            'campaign.dynamic_search_ads_setting.language_code',
                            'campaign.start_date_time',
                            'campaign.end_date_time',
                            'campaign.status',
                            'campaign.manual_cpm',
                            'campaign.manual_cpv',
                            'campaign.maximize_conversion_value.target_roas',
                            'campaign.maximize_conversions.target_cpa_micros',
                            'campaign.network_settings.target_content_network',
                            'campaign.network_settings.target_google_search',
                            'campaign.network_settings.target_partner_search_network',
                            'campaign.network_settings.target_search_network',
                            'campaign.optimization_goal_setting.optimization_goal_types',
                            'campaign.optimization_score',
                            'campaign.payment_mode',
                            'campaign.serving_status',
                            'campaign.shopping_setting.campaign_priority',
                            'campaign.target_roas.target_roas',
                            'campaign.tracking_url_template',
                            'customer.descriptive_name',
                            'customer.id'),
  where                 = NULL,
  order_by              = NULL,
  limit                 = NULL,
  parameters            = NULL,
  customer_id           = getOption('gads.customer.id'),
  login_customer_id     = getOption('gads.login.customer.id'),
  include_resource_name = FALSE,
  cl                    = NULL,
  verbose               = TRUE
) {

  # user gads_get_report
  res <- gads_get_report(
    resource          = 'campaign',
    fields            = fields,
    where             = where,
    order_by          = order_by,
    limit             = limit,
    parameters        = parameters,
    customer_id       = customer_id,
    login_customer_id = login_customer_id,
    verbose           = verbose,
    date_from         = NULL,
    date_to           = NULL,
    cl                = cl
  )

  # renaming to snale case
  res <- rename_with(res, function(x) str_remove( str_to_lower(x), 'campaign\\_?'), matches('campaign', ignore.case = TRUE) ) %>%
         rename_with(getOption('gads.column.name.case.fun'))

  # fix date
  if ( any(str_detect(str_to_lower(names(res)), 'date')) ) {
    res <- mutate(res,
                  across(matches('date', ignore.case = TRUE), as.Date))
  }

}

#' Get resource or field information.
#'
#' @return List of resource or field metadata
#' @param object_name name of resource, resource's field, segmentation field or metric
#' @export
#'
#' @seealso \href{https://developers.google.com/google-ads/api/docs/concepts/field-service}{Resource Metadata API documentation}
#'
#' @examples
#' \dontrun{
#' ad_group_info <- gads_get_fields("ad_group")
#' }
gads_get_fields <- function(
  object_name
)
{

  # check token
  gargle::token_tokeninfo(gads_token())

  # send query
  out <- request_build(
    method   = "GET",
    path     = str_glue('{options("gads.api.version")}/googleAdsFields/{object_name}'),
    token    = gads_token(),
    base_url = getOption('gads.base.url')
  )

  # send request
  ans <- request_retry(
    out,
    add_headers(`developer-token`= gads_developer_token())
  )

  # get result
  rawres <- response_process(ans, error_message = gads_check_errors2)

  # rq id
  rq_ids <- unique(ans$headers$`request-id`)
  rgoogleads$last_request_id <- rq_ids

  # check for error
  gads_check_errors(out = rawres, client_id = object_name, request_id = rq_ids)

  # parsing
  for ( x in names(rawres) ) {
    if (is.list(rawres[[x]])) rawres[[x]] <- unlist(rawres[[x]])
  }

  # success msg
  cli_alert_success('Success!')

  # return data
  return(rawres)
}

#' Get Google Ads Manager Account Hierarchy
#'
#' @param manager_customer_id ID of the manager account whose hierarchy you want to get.
#' @param include_drafts logical, Incliding drafts child account.
#' @param login_customer_id Ypor top-level manager account id.
#'
#' @return tibble with data of all the child accounts
#' @seealso \href{https://developers.google.com/google-ads/api/docs/account-management/get-account-hierarchy}{Get Account Hierarchy API documentation}
#' @export
#'
#' @examples
#' \dontrun{
#' acc_hier <- gads_get_account_hierarchy(
#'     manager_customer_id = '111-111-1111',
#'     login_customer_id   = '000-000-0000')
#' }
gads_get_account_hierarchy <- function(
  manager_customer_id = getOption('gads.login.customer.id'),
  include_drafts      = FALSE,
  login_customer_id   = getOption('gads.login.customer.id')
) {

  # remove - from manager_customer id
  manager_customer_id <- str_replace_all(manager_customer_id, '-', '')

  # build GAQL Query
  gaql_query <- gads_make_query(
    resource = 'customer_client',
    fields = c( 'customer_client.applied_labels',
                'customer_client.client_customer',
                'customer_client.currency_code',
                'customer_client.descriptive_name',
                'customer_client.hidden',
                'customer_client.id',
                'customer_client.level',
                'customer_client.manager',
                'customer_client.test_account',
                'customer_client.time_zone',
                'customer.currency_code',
                'customer.descriptive_name',
                'customer.final_url_suffix',
                'customer.has_partners_badge',
                'customer.id',
                'customer.manager',
                'customer.optimization_score',
                'customer.optimization_score_weight',
                'customer.pay_per_conversion_eligibility_failure_reasons',
                'customer.test_account',
                'customer.time_zone',
                'customer.tracking_url_template'
      ),date_from = NULL, date_to = NULL, during = NULL,
    parameters = ifelse(include_drafts, 'include_drafts=true', 'include_drafts=false')
  )

  body <- list(query = gaql_query) %>%
          toJSON(auto_unbox = T, pretty = T)

  # send query
  out <- gads_make_request(
    http_method = "POST",
    body = body,
    api_method = str_glue('customers/{manager_customer_id}/googleAds:searchStream')
  )

  # parse answer
  tibble(data = out) %>%
    unnest_wider('data') %>%
    select('results') %>%
    unnest_longer('results') %>%
    unnest_wider('results') %>%
    unnest_wider('customer') %>%
    unnest_wider('customerClient', names_sep = "_") %>%
    rename_with(getOption('gads.column.name.case.fun')) -> res

  # success msg
  cli_alert_success('Success! Loaded {nrow(res)} rows!')

  # return result
  return(res)

}

gads_get_report_helper <- function(
  gaql_query,
  customer_id           = getOption('gads.customer.id'),
  login_customer_id     = getOption('gads.login.customer.id'),
  include_resource_name = FALSE,
  verbose               = TRUE
) {

  if (verbose) message('---------------------------')
  # delete _
  customer_id <- str_replace_all(customer_id, '-', '')
  login_customer_id <- str_replace_all(login_customer_id, '-', '')

  # manager_customer id
  login_customer_id <- ifelse(length(login_customer_id) == 0, customer_id, login_customer_id)

  # check login and custimer ids
  if ( is.null(customer_id) ) {
    gads_abort('customer_id is require argument, please set it and retry your request.')
  }

  # info
  if (verbose) cli_alert_info(c('Loading data: ', str_replace(customer_id, '(\\d{3})(\\d{3})(\\d{4})', '\\1-\\2-\\3')))

  # info
  if (verbose) cli_alert_info('Compose query')

  # --------------
  # build GAQL Query
  body <- list(query = gaql_query) %>%
          toJSON(auto_unbox = T, pretty = T)

  # --------------
  # info
  if (verbose) cli_alert_info('Send request')

  out <- gads_make_request(
    body              = body,
    api_method        = str_glue("customers/{customer_id}/googleAds:searchStream"),
    customer_id       = customer_id,
    login_customer_id = login_customer_id,
    verbose           = verbose
    )

  # check for errors
  #gads_check_errors(out, customer_id, verbose, rq_ids)

  # empty answer handler
  if ( length(out) == 0 ) return(tibble() )

  # --------------
  # parsing answer
  # result df object
  res <- list()

  # get result
  if (verbose) cli_alert_info('Parsing result')
  for ( result in out ) {

    # get parts of answer
    parts_name <- names(result$results[[1]])

    # start unnesting - top level
    temp <- tibble(data = result$results) %>%
            unnest_wider('data')

      # unnest all parts
      for ( part in parts_name ) {

        temp <- unnest_wider(temp, col = part, names_sep = '_')

      }

    # add to result list
    res <- append(res, list(temp))
  }

  # binding
  res <- bind_rows(res)

  # detect other list columns
  list_col <- sapply(res, class)[sapply(res, class, USE.NAMES = F) == "list"]

  # unnesting each list column
  for ( col in names(list_col) ) {

    res <- unnest_wider(res, col, names_sep = "_")

  }

  # convert metrics to numeric
  if (verbose) cli_alert_info('Convert metrics to numeric type')
  res <- mutate(res,
                across(matches('metrics'), as.numeric)
  )

  # renaming to snale case
  if (verbose) cli_alert_info('Rename columns to gads.column.name case')
  res <- rename_with(res, getOption('gads.column.name.case.fun')) %>%
         rename_with(gads_fix_names, matches('metrics|segments', ignore.case = TRUE) )


  # fix date
  if ( any(str_detect(str_to_lower(names(res)), 'date')) ) {
    if (verbose) cli_alert_info('Fix date fields')
    try( {
      res <- mutate(res,
                    across(matches('date', ignore.case = TRUE) & !matches('interval', ignore.case = TRUE), as.Date))
    },
    silent = TRUE)
  }

  # fix cost
  if ( any(str_detect(str_to_lower(names(res)), 'micros')) ) {
    if (verbose) cli_alert_info('Fix cost fields')

    res <- mutate(res,
                  across(matches('micros', ignore.case = TRUE), function(x) round(as.numeric(x) / 1000000, 2 )) ) %>%
           rename_with(gads_fix_names_regexp, matches('micros', ignore.case = TRUE), regexp = "\\_micros|micros")

  }

  # remove resource names
  if ( isFALSE(include_resource_name) ) {

    res <- select(res, -matches('resource\\_?name'))

  }

  # success msg
  if (verbose) cli_alert_success('Success! Loaded {nrow(res)} rows!')

  # return
  return(res)

}

#' Get data from Google Ads API
#'
#' @param resource Report type, you can get list of all acessible resource using \code{\link{gads_get_metadata}}. For more information see \href{https://developers.google.com/google-ads/api/fields/v23/overview#list-of-all-resources}{link with list of all resources}
#' @param fields character vector, list of report fields, all report has own fields list. You can get list of accesible resource fields using \code{\link{gads_get_fields}} for example \href{https://developers.google.com/google-ads/api/fields/v23/campaign}{see field list of campaign report}.
#' @param where Filter, for example you can filter campaigns by status \code{where = "campaign.status = 'ENABLED'"}.
#' @param order_by Sorting, character vectors of fields and sorting directions, for example \code{order_by = c("campaign.name DESC", "metrics.clicks")}.
#' @param limit Maximun rows in report
#' @param parameters Query parameters, for example \code{parameters = "include_drafts=true"}.
#' @param date_from Beginning of date range. Format: 2018-01-01
#' @param date_to End of date rage. Format: 2018-01-10
#' @param during Predefined date range. See \href{https://developers.google.com/google-ads/api/docs/query/date-ranges#predefined_date_range}{documentation} for more details.
#' @param customer_id Google Ads client customer id, supports a single account id: "xxx-xxx-xxxx" or a vector of ids from the same Google Ads MCC: c("xxx-xxx-xxxx", "xxx-xxx-xxxx")
#' @param login_customer_id Google Ads manager customer id
#' @param include_resource_name Get resource names fields in report
#' @param gaql_query GAQL Query, you can make it in \code{\link{gads_get_metadata}}. For more information see \href{https://developers.google.com/google-ads/api/fields/v23/overview_query_builder}{Query Builder}. If you use gaql_query, you don't need set other query parameters like resource, fields, where, dates etc.
#' @param verbose Console log output
#' @param cl A cluster object created by \code{\link{makeCluster}}, or an integer to indicate number of child-processes (integer values are ignored on Windows) for parallel evaluations (see Details on performance).
#'
#' @return tibble with the Google Ads Data.
#' @seealso
#' \itemize{
#'   \item \href{https://developers.google.com/google-ads/api/fields/v23/overview}{Oficial Google Ads API Reports documantation}
#'   \item \href{https://developers.google.com/google-ads/api/fields/v23/overview_query_builder}{Google Ads Query Builder}
#' }
#' @export
#'
#' @examples
#' \dontrun{
#' # set client id
#' gads_set_login_customer_id('xxx-xxx-xxxx')
#'
#' # set manager id if you work under MCC
#' gads_set_customer_id('xxx-xxx-xxxx')
#'
#' # default paramas is campaign performance report
#' campaign_stat <- gads_get_report()
#'
#'
#' # you can load data from several client accounts at once
#' # from the same Google Ads MCC
#' # client ids
#' accounts <- c('xxx-xxx-xxxx', 'yyy-yyy-yyyy')
#' # loading data
#' multi_rep <- gads_get_report(
#'     date_from = as.Date('2021-06-10'),
#'     date_to = as.Date('2021-06-17'),
#'     customer_id = accounts
#' )
#'
#' # ------------------
#' # using more arguments for other reports
#' group_report <- gads_get_report(
#' customer_id = 4732519773,
#' resource    = "ad_group",
#' fields = c("ad_group.campaign",
#'            "ad_group.id",
#'            "ad_group.name",
#'            "ad_group.status",
#'            "metrics.clicks",
#'            "metrics.cost_micros"),
#' date_from   = "2021-06-10",
#' date_to     = "2021-06-17",
#' where       = "ad_group.status = 'ENABLED'",
#' order_by    = c("metrics.clicks DESC", "metrics.cost_micros"),
#' limit       = 30000
#' )
#'
#' # ------------------
#' # parallel loading mode
#' # note: you must using login_customer_id agrument in parallel mode
#' # because oprions gads_set_login_customer_id() does't work in parallel mode loading
#' library(parallel)
#'
#' # make core cluster
#' cl <- makeCluster(4)
#'
#' # loading data
#' multi_rep <- gads_get_report(
#'   date_from         = as.Date('2021-06-10'),
#'   date_to           = as.Date('2021-06-17'),
#'   customer_id       = c('111-111-1111',
#'                         '222-222-2222',
#'                         '333-333-3333',
#'                         '444-444-4444',
#'                         '555-555-5555'),
#'   login_customer_id = "999-999-9999",
#'   cl                = cl
#' )
#'
#' # stop cluster
#' stopCluster(cl)
#' }
gads_get_report <- function(
  resource              = 'campaign',
  fields = c('campaign.id',
             'campaign.name',
             'customer.id',
             'customer.descriptive_name',
             'campaign.status',
             'segments.date',
             'metrics.all_conversions',
             'metrics.clicks',
             'metrics.cost_micros',
             'metrics.ctr',
             'metrics.impressions',
             'metrics.interaction_rate',
             'metrics.interactions',
             'metrics.invalid_clicks'),
  where                 = NULL,
  order_by              = NULL,
  limit                 = NULL,
  parameters            = NULL,
  date_from             = Sys.Date() - 15,
  date_to               = Sys.Date() - 1,
  during                = c(NA, "TODAY", "YESTERDAY", "LAST_7_DAYS", "LAST_BUSINESS_WEEK", "THIS_MONTH", "LAST_MONTH", "LAST_14_DAYS", "LAST_30_DAYS", "THIS_WEEK_SUN_TODAY", "THIS_WEEK_MON_TODAY", "LAST_WEEK_SUN_SAT", "LAST_WEEK_MON_SUN"),
  customer_id           = getOption('gads.customer.id'),
  login_customer_id     = getOption('gads.login.customer.id'),
  include_resource_name = FALSE,
  gaql_query            = NULL,
  cl                    = NULL,
  verbose               = TRUE
) {

  # test for selectble with date fields
  selectable <- suppressMessages( gads_get_fields_cached(resource)$selectableWith )

  if ( getOption('gads.show_gaql_query') ) cat("\n\nGAQL Query:\n\n", gaql_query)

  # where block
  if (! "segments.date" %in% selectable ) {
    date_from <- NULL
    date_to   <- NULL
    during    <- NULL
    cli_alert_warning('fields values of date_from, date_to and during was unset automatically, because it is not selectable with {resource}')
  }

  # query
  # compose query if needed
  if ( is.null(gaql_query) ) {
    gaql_query <- gads_make_query(
      resource,
      fields,
      where,
      order_by,
      limit,
      parameters,
      date_from,
      date_to,
      during
    )
  }

  # check how many accounts
  if ( length(customer_id) == 1 ) {

    # if only one account use gads_get_report
    res <- gads_get_report_helper(
      gaql_query            = gaql_query,
      customer_id           = customer_id,
      login_customer_id     = login_customer_id,
      include_resource_name = include_resource_name,
      verbose               = verbose)

  } else {

    cli_alert_info('Multi account request')

    # check parallel
    if (!is.null(cl)) {

      # check
      if ( is.null(login_customer_id) ) gads_abort('You have enabled multi-threaded loading, and you did not specify the login_customer_id argument, which is required to loading data in multi-threaded mode.')

      # optins
      oldpar <- options('gads.multi.account.verbos')
      on.exit(options(oldpar))
      cli_alert_info('Verbose mode is deactive because you run loading in parallel mode!')
      options('gads.multi.account.verbos' = FALSE)
    }

    # define apply function
    if (getOption('gads.multi.account.verbos')) {

      cli_alert_info('Verbose mode is active for multi account loading!')
      cli_alert_info('start------------------------------------------->')

      mfun    <- 'lapply'
      verbose <- TRUE

      mfun_args <- list(
        customer_id,
        safely(
          function(x) {
            gads_get_report_helper(
              gaql_query            = gaql_query,
              customer_id           = x,
              login_customer_id     = login_customer_id,
              include_resource_name = include_resource_name,
              verbose = verbose)
          }
        )
      )


    } else {

      mfun    <- 'pblapply'
      verbose <- FALSE

      mfun_args <- list(
        customer_id,
        safely(
          function(x) {
            gads_get_report_helper(
              gaql_query            = gaql_query,
              customer_id           = x,
              login_customer_id     = login_customer_id,
              include_resource_name = include_resource_name,
              verbose = verbose)
          }),
        cl = cl
      )

    }

    # run function
    res <- do.call(mfun, mfun_args)

    # get res and errors
    res <- transpose(res)

    # check errors
    if ( length(res$error) > 0 ) {

      for ( err in res$error ) {

        if ( is.null(err) ) next

        cli_alert_danger(err$message)

      }

    }

    # check
    res_n <- list.filter(res$result, !is.null(.)) %>% length()
    err_n <- list.filter(res$error, !is.null(.)) %>% length()

    # bind
    res <- bind_rows(res$result)

    # res
    if (getOption('gads.multi.account.verbos') & verbose) cli_alert_info('end--------------------------------------------->')

    # number off error
    if (err_n > 0) {
      cli_alert_danger('Data loading was unsuccessful for {err_n} accounts.')
    }

    # check result
    if (nrow(res) == 0) {
      cli_alert_warning('The request you sent did not return any results, check the entered parameters and repeat the opposition.')
    } else if (res_n > err_n) {
      cli_alert_success('Success! Loaded {nrow(res)} rows!')
    } else if (res_n == err_n) {
      cli_alert_danger('!!! LOADING ERROR: see previos messages for details information')
    }

  }

  return(res)

}

## This file is the interface between bigrquery and the
## auth functionality in gargle.

# initialization happens in .onLoad
.auth <- NULL
app <- NULL

## The roxygen comments for these functions are mostly generated from data
## in this list and template text maintained in gargle.
gargle_lookup_table <- list(
  PACKAGE     = "rgoogleads",
  YOUR_STUFF  = "your Google Ads Account",
  PRODUCT     = "Google Ads",
  API         = "Google Ads API",
  PREFIX      = "gads"
)

# main oauth function
#' Authorization in Google Ads API
#'
#'
#' @param email Optional. Allows user to target a specific Google identity.
#' @param path Path to JSON file with identifying the service account
#' @param cache Specifies the OAuth token cache.
#' @param use_oob Whether to prefer "out of band" authentication.
#' @param developer_token Your Google Ads Developer Token.
#' @param token A token with class \link[httr:Token-class]{Token2.0} or an object of
#' @eval gargle:::PREFIX_auth_description(gargle_lookup_table)
#' @eval gargle:::PREFIX_auth_details(gargle_lookup_table)
#' @eval gargle:::PREFIX_auth_params()
#' @family auth functions
#'
#' @return \link[httr:Token-class]{Token2.0}
#' @export
#'
#' @examples
#' \dontrun{
#' ## load/refresh existing credentials, if available
#' ## otherwise, go to browser for authentication and authorization
#' gads_auth()
#'
#' ## force use of a token associated with a specific email
#' gads_auth(email = "yourname@example.com")
#'
#' ## force a menu where you can choose from existing tokens or
#' ## choose to get a new one
#' gads_auth(email = NA)
#'
#' ## -----------------------
#' ## use own developer token
#' gads_auth(
#'     email = "yourname@example.com",
#'     developer_token = "your developer token"
#' )
#'
#' ## -----------------------
#' ## use own OAuth client app
#' gads_auth_configure(
#'     path = "path/to/your/oauth_client.json"
#' )
#'
#' gads_auth(email = "yourname@example.com")
#' }
gads_auth <- function(
  email           = gargle::gargle_oauth_email(),
  path            = NULL,
  cache           = gargle::gargle_oauth_cache(),
  use_oob         = gargle::gargle_oob_default(),
  developer_token = getOption('gads.developer.token'),
  token           = NULL) {

  # check default app
  app <- gads_oauth_app() %||% gads_default_ouath_app()

  # check link between app_secret and developer token
  if ( app$secret == "302158242268-eqkksdns6gbdl7qf0v59639pder9knql.apps.googleusercontent.com" & developer_token != "EBkkx-znu2cZcEY7e74smg" ) {
    gads_abort("You can`t use default oauth app with own developer token, please create default app and set it by gads_auth_configure()")
  }

  cred <- gargle::token_fetch(
    scopes  = 'https://www.googleapis.com/auth/adwords',
    app     = app,
    email   = email,
    path    = path,
    package = "rgoogleads",
    cache   = cache,
    use_oob = use_oob,
    token   = token,
    user_params = list(developer_token = developer_token)
  )

  if (!inherits(cred, "Token2.0")) {
    stop(
      "Can't get Google credentials.\n",
      "Are you running rgoogleads in a non-interactive session? Consider:\n",
      "  * Call `gads_auth()` directly with all necessary specifics.\n",
      call. = FALSE
    )
  }
  .auth$set_cred(cred)
  .auth$set_auth_active(TRUE)

  invisible()
}

#' Suspend authorization
#' @eval gargle:::PREFIX_deauth_description_with_api_key(gargle_lookup_table)
#' @family auth functions
#' @return only suspend authorization
#' @export
gads_deauth <- function() {
  .auth$set_auth_active(FALSE)
  .auth$clear_cred()

  invisible()
}

#' Produce configured token
#'
#' @eval gargle:::PREFIX_token_description(gargle_lookup_table)
#' @eval gargle:::PREFIX_token_return()
#' @family low-level API functions
#' @export
gads_token <- function() {
  if (isFALSE(.auth$auth_active)) {
    return(NULL)
  }
  if (!gads_has_token()) {
    gads_auth()
  }
  httr::config(token = .auth$cred)
}

# has token
#' Is there a token on hand?
#'
#' @eval gargle:::PREFIX_has_token_description(gargle_lookup_table)
#' @eval gargle:::PREFIX_has_token_return()
#' @family low-level API functions
#' @export
gads_has_token <- function() {
  inherits(.auth$cred, "Token2.0")
}


#' Edit and view auth configuration
#'
#' @eval gargle:::PREFIX_auth_configure_description(gargle_lookup_table)
#' @eval gargle:::PREFIX_auth_configure_params()
#' @eval gargle:::PREFIX_auth_configure_return(gargle_lookup_table)
#' @param developer_token Your Google Ads Developer Token.
#'
#' @family auth functions
#' @export
#' @examples
#' \dontrun{
#' # see and store the current user-configured OAuth app (probaby `NULL`)
#' (original_app <- gads_oauth_app())
#'
#' # see and store the current user-configured API key (probaby `NULL`)
#' (original_api_key <- gads_api_key())
#'
#' if (require(httr)) {
#'   # bring your own app via client id (aka key) and secret
#'   google_app <- httr::oauth_app(
#'     "my-awesome-google-api-wrapping-package",
#'     key = "YOUR_CLIENT_ID_GOES_HERE",
#'     secret = "YOUR_SECRET_GOES_HERE"
#'   )
#'   google_key <- "YOUR_API_KEY"
#'   gads_auth_configure(app = google_app, api_key = google_key)
#'
#'   # confirm the changes
#'   gads_oauth_app()
#'   gads_api_key()
#'
#'   # bring your own app via JSON downloaded from Google Developers Console
#'   # this file has the same structure as the JSON from Google
#'   gads_auth_configure(path = app_path)
#'
#'   # confirm the changes
#'   gads_oauth_app()
#'
#'   # use own developer token
#'   gads_auth_configure(developer_token = 'Your developer token')
#'
#' }
#'
#' # restore original auth config
#' gs4_auth_configure(app = original_app, api_key = original_api_key)
#' }
#' @export
gads_auth_configure <- function(client, path, api_key, developer_token, app = lifecycle::deprecated()) {

  if (lifecycle::is_present(app)) {
    gads_auth_configure(client = app, path = path, api_key = api_key)
  }

  if (!missing(client) && !missing(path)) {
    gads_abort("Must supply exactly one of {.arg client} or {.arg path}, not both")
  }

  stopifnot(missing(api_key) || is.null(api_key) || is_string(api_key))

  if (!missing(path)) {
    stopifnot(is_string(path))
    client <- gargle::oauth_app_from_json(path)
  }
  stopifnot(missing(client) || is.null(client) || inherits(client, "oauth_app"))

  if (!missing(client) || !missing(path)) {
    .auth$set_app(client)
  }

  if (!missing(api_key)) {
    .auth$set_api_key(api_key)
  }

  if (!missing(developer_token)) {
    options('gads.developer.token' = developer_token)
  }

  invisible(.auth)
}


#' @export
#' @rdname gads_auth_configure
gads_auth_cache_path <- function() {

  if ( gads_has_token() ) {
    .auth$cred$cache_path
  } else {
    cli_alert_warning("You need to log in to google account")
  }

}

#' Open folder with auth cache files
#' @export
#' @rdname gads_auth_configure
gads_open_auth_cache_folder <- function() {

  if ( gads_has_token() ) {
    shell.exec(.auth$cred$cache_path)
  } else {
    cli_alert_warning("You need to log in to google account")
  }

}

# gads abort
gads_abort <- function(message, ..., .envir = parent.frame()) {
  cli::cli_div(theme = gads_theme())
  cli::cli_abort(message = message, ..., .envir = .envir)
}

# is_path is_string
is_path <- function(x) is.character(x) && !inherits(x, "drive_id")
is_string <- function(x) length(x) == 1L && is_path(x)

# theme
gads_theme <- function() {
  list(
    span.field = list(transform = single_quote_if_no_color),
    # I want to style the Drive file names similar to cli's `.file` style,
    # except cyan instead of blue
    span.drivepath = list(
      color = "cyan",
      fmt = utils::getFromNamespace("quote_weird_name", "cli")
    ),
    # since we're using color so much elsewhere, e.g. Drive file names, I think
    # the standard bullet should be "normal" color
    ".memo .memo-item-*" = list(
      "text-exdent" = 2,
      before = function(x) paste0(cli::symbol$bullet, " ")
    )
  )
}

is_testing <- function() {
  identical(Sys.getenv("TESTTHAT"), "true")
}

single_quote_if_no_color <- function(x) quote_if_no_color(x, "'")

quote_if_no_color <- function(x, quote = "'") {
  # TODO: if a better way appears in cli, use it
  # @gabor says: "if you want to have before and after for the no-color case
  # only, we can have a selector for that, such as:
  # span.field::no-color
  # (but, at the time I write this, cli does not support this yet)
  if (cli::num_ansi_colors() > 1) {
    x
  } else {
    paste0(quote, x, quote)
  }
}

# getters
#' @export
#' @rdname gads_auth_configure
gads_api_key <- function() .auth$api_key

#' @export
#' @rdname gads_auth_configure
gads_developer_token <- function() {

  dev_token <- .auth$cred$params$user_params$developer_token

  if ( is.null(dev_token) ) {
    dev_token <- getOption('gads.developer.token')
  }

  return(dev_token)

}


#' @export
#' @rdname gads_auth_configure
gads_oauth_app <- function() .auth$app

#' Get info on current user
#'
#' @eval gargle:::PREFIX_user_description()
#' @eval gargle:::PREFIX_user_seealso()
#' @eval gargle:::PREFIX_user_return()
#'
#' @export
gads_user <- function() {
  if (gads_has_token()) {
    gargle::token_email(gads_token())
  } else {
    cli_alert_info("Not logged in as any specific Google user")
    invisible()
  }
}


# default app
gads_default_ouath_app <- function() {

  app <- httr::oauth_app(
    'rgoogleads',
    '302158242268-eqkksdns6gbdl7qf0v59639pder9knql.apps.googleusercontent.com',
    'l5_BBLUcd-LFkWHdnjuS9ACt'
    )

  return(app)

}

# local deauth
local_deauth <- function(env = parent.frame()) {
  original_cred <- .auth$get_cred()
  original_auth_active <- .auth$auth_active
  gads_bullets(c("i" = "Going into deauthorized state"))
  withr::defer(
    gads_bullets(c("i" = "Restoring previous auth state")),
    envir = env
  )
  withr::defer({
    .auth$set_cred(original_cred)
    .auth$set_auth_active(original_auth_active)
  }, envir = env)
  gads_deauth()
}

gads_bullets <- function(text, .envir = parent.frame()) {
  quiet <- gads_quiet()
  if (quiet) {
    return(invisible())
  }
  cli::cli_div(theme = gads_theme())
  cli::cli_bullets(text = text, .envir = .envir)
}

gads_quiet <- function() {
  getOption("gads_quiet", default = NA)
}


#' Set manager customer id in current R session
#'
#' @param customer_id your manager customer id
#'
#' @return only set options
#' @export
gads_set_login_customer_id <- function(customer_id) {
  # delete _
  customer_id <- str_replace_all(customer_id, '-', '')
  # set option
  options(gads.login.customer.id = customer_id)
  # info
  out_msg_id <- gsub("(\\d{3})(\\d{3})(\\d{4})", "\\1-\\2-\\3", customer_id)
  cli_alert_info(str_glue('You set login_customer_id: {out_msg_id}'))
}


#' Set client customer id in current R session
#'
#' @param customer_id your client customer id
#'
#' @return only set options
#' @export
gads_set_customer_id <- function(customer_id) {
  # delete _
  customer_id <- str_replace_all(customer_id, '-', '')
  # set option
  options(gads.customer.id = customer_id)
  # info
  out_msg_id <- gsub("(\\d{3})(\\d{3})(\\d{4})", "\\1-\\2-\\3", customer_id)
  cli_alert_info(str_glue('You set customer_id: {str_c(out_msg_id, collapse = ", ")}'))
}

#' Get Ads Dictionary From Google Ads Client Account
#'
#' @inheritParams gads_get_report
#' @param fields character vector, list pf report fields, all report has own fields list, for example \href{https://developers.google.com/google-ads/api/fields/v23/ad_group_ad}{see field list of ads report}.
#'
#' @seealso \href{https://developers.google.com/google-ads/api/fields/v23/ad_group_ad_query_builder}{Google Ads Query Builder}
#' @return tibble with ads dicrionary
#' @export
#' @examples
#' \dontrun{
#' # set client customer id
#' gads_set_login_customer_id('xxx-xxx-xxxx')
#'
#' # set manager id if you work under MCC
#' gads_set_customer_id('xxx-xxx-xxxx')
#'
#' # load ads list
#' myads <- gads_get_ads(
#'    fields = c("ad_group_ad.ad.id",
#'               "customer.descriptive_name"),
#'    where = 'ad_group_ad.status = "ENABLED"'
#' )
#'
#' }
gads_get_ads <- function(
  fields                = c('ad_group_ad.ad.id',
                            'ad_group_ad.ad.name',
                            'ad_group_ad.ad.added_by_google_ads',
                            'ad_group_ad.ad.app_ad.descriptions',
                            'ad_group_ad.ad.app_ad.headlines',
                            'ad_group_ad.ad.app_ad.html5_media_bundles',
                            'ad_group_ad.ad.app_ad.images',
                            'ad_group_ad.ad.app_ad.mandatory_ad_text',
                            'ad_group_ad.ad.app_engagement_ad.videos',
                            'ad_group_ad.ad.device_preference',
                            'ad_group_ad.ad.display_upload_ad.display_upload_product_type',
                            'ad_group_ad.ad.display_upload_ad.media_bundle',
                            'ad_group_ad.ad.display_url',
                            'ad_group_ad.ad.expanded_dynamic_search_ad.description',
                            'ad_group_ad.ad.expanded_dynamic_search_ad.description2',
                            'ad_group_ad.ad.expanded_text_ad.description',
                            'ad_group_ad.ad.expanded_text_ad.description2',
                            'ad_group_ad.ad.expanded_text_ad.headline_part1',
                            'ad_group_ad.ad.expanded_text_ad.headline_part2',
                            'ad_group_ad.ad.expanded_text_ad.headline_part3',
                            'ad_group_ad.ad.expanded_text_ad.path1',
                            'ad_group_ad.ad.expanded_text_ad.path2',
                            'ad_group_ad.ad.final_url_suffix',
                            'ad_group_ad.ad.final_urls',
                            'ad_group_ad.ad.final_mobile_urls',
                            'ad_group_ad.ad.hotel_ad',
                            'ad_group_ad.ad.image_ad.image_url',
                            'ad_group_ad.ad.image_ad.mime_type',
                            'ad_group_ad.ad.image_ad.name',
                            'ad_group_ad.ad.image_ad.pixel_height',
                            'ad_group_ad.ad.image_ad.pixel_width',
                            'ad_group_ad.ad.image_ad.preview_image_url',
                            'ad_group_ad.ad.image_ad.preview_pixel_height',
                            'ad_group_ad.ad.image_ad.preview_pixel_width',
                            'ad_group_ad.ad.legacy_app_install_ad',
                            'ad_group_ad.ad.legacy_responsive_display_ad.accent_color',
                            'ad_group_ad.ad.legacy_responsive_display_ad.allow_flexible_color',
                            'ad_group_ad.ad.legacy_responsive_display_ad.business_name',
                            'ad_group_ad.ad.legacy_responsive_display_ad.description',
                            'ad_group_ad.ad.legacy_responsive_display_ad.call_to_action_text',
                            'ad_group_ad.ad.legacy_responsive_display_ad.format_setting',
                            'ad_group_ad.ad.legacy_responsive_display_ad.logo_image',
                            'ad_group_ad.ad.legacy_responsive_display_ad.long_headline',
                            'ad_group_ad.ad.legacy_responsive_display_ad.main_color',
                            'ad_group_ad.ad.legacy_responsive_display_ad.marketing_image',
                            'ad_group_ad.ad.legacy_responsive_display_ad.price_prefix',
                            'ad_group_ad.ad.legacy_responsive_display_ad.promo_text',
                            'ad_group_ad.ad.legacy_responsive_display_ad.short_headline',
                            'ad_group_ad.ad.legacy_responsive_display_ad.square_logo_image',
                            'ad_group_ad.ad.legacy_responsive_display_ad.square_marketing_image',
                            'ad_group_ad.ad.local_ad.call_to_actions',
                            'ad_group_ad.ad.local_ad.descriptions',
                            'ad_group_ad.ad.local_ad.headlines',
                            'ad_group_ad.ad.local_ad.logo_images',
                            'ad_group_ad.ad.local_ad.marketing_images',
                            'ad_group_ad.ad.local_ad.path1',
                            'ad_group_ad.ad.local_ad.path2',
                            'ad_group_ad.ad.resource_name',
                            'ad_group_ad.ad.responsive_display_ad.accent_color',
                            'ad_group_ad.ad.responsive_display_ad.allow_flexible_color',
                            'ad_group_ad.ad.responsive_display_ad.business_name',
                            'ad_group_ad.ad.responsive_display_ad.call_to_action_text',
                            'ad_group_ad.ad.responsive_display_ad.control_spec.enable_asset_enhancements',
                            'ad_group_ad.ad.responsive_display_ad.control_spec.enable_autogen_video',
                            'ad_group_ad.ad.responsive_display_ad.format_setting',
                            'ad_group_ad.ad.responsive_display_ad.headlines',
                            'ad_group_ad.ad.responsive_display_ad.long_headline',
                            'ad_group_ad.ad.responsive_display_ad.main_color',
                            'ad_group_ad.ad.responsive_display_ad.price_prefix',
                            'ad_group_ad.ad.responsive_display_ad.promo_text',
                            'ad_group_ad.ad.responsive_display_ad.square_marketing_images',
                            'customer.descriptive_name',
                            'customer.id'),
  where                 = NULL,
  order_by              = NULL,
  limit                 = NULL,
  parameters            = NULL,
  customer_id           = getOption('gads.customer.id'),
  login_customer_id     = getOption('gads.login.customer.id'),
  include_resource_name = FALSE,
  cl                    = NULL,
  verbose               = TRUE
) {

  # user gads_get_report
  res <- gads_get_report(
    resource          = 'ad_group_ad',
    fields            = fields,
    where             = where,
    order_by          = order_by,
    limit             = limit,
    parameters        = parameters,
    customer_id       = customer_id,
    login_customer_id = login_customer_id,
    verbose           = verbose,
    date_from         = NULL,
    date_to           = NULL,
    cl                = cl
  )

  # fix names
  res <- rename_with(res, function(x) str_remove( str_to_lower(x), 'ad_group_ad\\_?'), matches('ad_group_ad', ignore.case = TRUE) ) %>%
         rename_with(getOption('gads.column.name.case.fun'))

}

#' Returns the requested Keyword Plan forecasts.
#'
#' @param keyword_plan_id Keyword plan id, you can get list of your keyword plans using \code{\link{gads_get_report}} with recource keyword_plan
#' @inheritParams gads_get_report
#'
#' @return tibble with keyword plan historical metrics
#' @export
#' @seealso \href{https://developers.google.com/google-ads/api/docs/keyword-planning/overview?hl=en}{Keyword Planning API Documentation}
#'
#' @examples
#' \dontrun{
#' # set client id
#' gads_set_customer_id('xxx-xxx-xxxx')
#'
#' # set manager id
#' gads_set_login_customer_id('xxx-xxx-xxxx')
#'
#' # get list of plan
#' plan_data <- gads_get_report(
#'   resource = 'keyword_plan',
#'   fields = c('keyword_plan.id')
#' )
#'
#' # get keyword historical data
#' historical_plan_data <- gads_keyword_plan_forecast_metrics(
#'  keyword_plan_id = plan_data$keyword_plan_id[1]#'
#' )
#'
#' }
gads_keyword_plan_forecast_metrics <- function(
  keyword_plan_id,
  customer_id       = getOption('gads.customer.id'),
  login_customer_id = getOption('gads.login.customer.id'),
  verbose           = TRUE
) {

  # delete - in customer id
  customer_id <- str_replace_all(customer_id, '-', '')

  # to env
  gads_customer_id_to_env(customer_id)

  # info
  if (verbose) cli_alert_info(c('Loading data: ', str_replace(customer_id, '(\\d{3})(\\d{3})(\\d{4})', '\\1-\\2-\\3')))

  # info
  if (verbose) cli_alert_info('Send query')

  # build query
  out <- request_build(
    method   = "POST",
    path     = str_glue('{options("gads.api.version")}/customers/{customer_id}/keywordPlans/{keyword_plan_id}:generateForecastMetrics'),
    token    = gads_token(),
    base_url = getOption('gads.base.url')
  )

  # send request
  ans <- request_retry(
    out,
    encode = "multipart",
    add_headers(`developer-token`= gads_developer_token(),
                `login-customer-id` = login_customer_id)
  )

  # request id
  rq_ids <- headers(ans)$`request-id`
  rgoogleads$last_request_id <- rq_ids

  # info
  if (verbose) cli_alert_info('Get response')

  # pars result
  data <- response_process(ans, error_message = gads_check_errors2)

  # info
  if (verbose) cli_alert_info('Parsing result')

  # camp data
  camp <- tibble(data = data$campaignForecasts) %>%
          unnest_wider('data') %>%
          unnest_wider('campaignForecast') %>%
          rename_with(getOption('gads.column.name.case.fun'))

  # adgroup data
  adgroup <- tibble(data = data$adGroupForecasts) %>%
             unnest_wider('data') %>%
             unnest_wider('adGroupForecast')%>%
             rename_with(getOption('gads.column.name.case.fun'))

  # camp data
  keyw <- tibble(data = data$keywordForecasts) %>%
          unnest_wider('data') %>%
          unnest_wider('keywordForecast')%>%
          rename_with(getOption('gads.column.name.case.fun'))

  # collect plan to one object
  res <- list(
    campaign_forecast = camp,
    ad_group_forecast = adgroup,
    keyword_forecast  = keyw
  )

  # fix cost
  if (verbose) cli_alert_info('Fix cost fields')
  for ( plan in names(res) ) {

    if ( any(str_detect( str_to_lower(names(res[[plan]])), 'micros|cpc')) ) {

      res[[plan]] <- mutate(res[[plan]],
                            across(matches('micros|cpc', ignore.case = TRUE), function(x) round(as.numeric(x) / 1000000, 2 )) ) %>%
                     rename_with(gads_fix_names_regexp, matches('micros', ignore.case = TRUE), regexp = "\\_micros|micros")

    }

  }

  # success msg
  if (verbose) cli_alert_success('Success!')
  if (verbose) cli_alert_info('For get data use res$campaign_forecast, res$ad_group_forecast and res$keyword_forecast')

  # return
  return(res)

}

#' Loading Data From 'Google Ads API'
#'
#' @description
#' \if{html}{\figure{rgoogleads.png}{options: style='float: right' alt='logo' width='120'}}
#' Interface for loading data from 'Google Ads API',
#' see <https://developers.google.com/google-ads/api/docs/start>.
#' Package provide function for authorization and loading reports.
#'
#' Capabilities of \code{rgoogleads}:
#' \itemize{
#'   \item Authorization in the Google Ads API
#'   \item Loading a list of top-level accounts
#'   \item Loading the entire hierarchy of accounts from manager accounts
#'   \item Loading list of Google Ads client account objects: campaigns, ad groups, ads, etc.
#'   \item Loading statistics from Google Ads client account
#'   \item Loading resource metadata, resource fields, segments and metrics
#'   \item Loading forecast and historical metrics from Keyword Planning.
#' }
#'
#' @examples
#' \dontrun{
#' library(rgoogleads)
#'
#' # set own oauth app
#' gads_auth_configure(path = 'C:/auth/app.json')
#' # set your developer token if needed, or use default developer token
#' gads_auth(email = 'me@gmail.com', developer_token = "own developer token")
#'
#' # get list of accessible accounts
#' my_accounts <- gads_get_accessible_customers()
#'
#' # set manager account id
#' gads_set_login_customer_id('xxx-xxx-xxxx')
#'
#' # set client account id
#' gads_set_customer_id('xxx-xxx-xxxx')
#'
#' # load report data
#' ad_group_report <- gads_get_report(
#'   resource    = "ad_group",
#'   fields = c("ad_group.campaign",
#'              "ad_group.id",
#'              "ad_group.name",
#'              "ad_group.status",
#'              "metrics.clicks",
#'              "metrics.cost_micros"),
#'   date_from   = "2021-06-10",
#'   date_to     = "2021-06-17",
#'   where       = "ad_group.status = 'ENABLED'",
#'   order_by    = c("metrics.clicks DESC", "metrics.cost_micros")
#' )
#' }
#'
#' @author Alexey Seleznev
#' @seealso
#' \itemize{
#'   \item \href{https://www.youtube.com/playlist?list=PLD2LDq8edf4qprTxRcflDwV9IvStiChHi}{YouTube lessons playlist}
#'   \item \href{https://developers.google.com/google-ads/api/docs/start}{Oficial Google Ads API documantation}
#'   \item \href{https://developers.google.com/google-ads/api/fields/v23/overview_query_builder}{Google Ads Query Builder}
#'   \item \href{https://selesnow.github.io/rgoogleads/docs/}{rgoogleads home page}
#' }
#'
#' @docType package
#' @name rgoogleads-package
#' @aliases rgoogleads
NULL

# fix no visible binding for global variable '.'
. <- NULL

#' Get all information about Google Ads Customer
#'
#' @param customer_id Google Ads customer id
#' @param verbose Processing log output into console
#'
#' @return Google Ads customer data
#'
#' @seealso \href{https://developers.google.com/google-ads/api/rest/common/search}{Method: SearchStream documentation}
#'
#' @export
gads_customer <- function(
  customer_id = getOption('gads.customer.id'),
  verbose = TRUE
) {

  # pars result
  data <- gads_get_report(
    resource          = "customer",
    fields            = c("customer.id",
                          "customer.descriptive_name",
                          "customer.manager",
                          "customer.currency_code",
                          "customer.time_zone",
                          "customer.auto_tagging_enabled",
                          "customer.has_partners_badge",
                          "customer.test_account"),
    customer_id       = customer_id,
    login_customer_id = customer_id,
    verbose           = verbose
  )

  data <- rename_with(data, gads_fix_names_regexp, everything(), regexp = 'customer\\_')

  # return the data
  return(data)
}

#' Get all data of customers directly accessible by the user authenticating the call.
#'
#' @return List of your accessible accounts from top level
#' @export
#'
#' @examples
#' \dontrun{
#' accounts <- gads_get_accessible_customers()
#' }
gads_get_accessible_customers <- function()
  {

  # send query
  rawres <- gads_make_request(
    http_method = "GET",
    api_method  = "customers:listAccessibleCustomers"
    )

  # processing result
  unlist(rawres$resourceNames) %>%
    str_replace_all('customers/', '') -> account_ids

  # info
  cli_alert_info(c("Your accessible accounts ids: ", str_replace_all(account_ids, '(\\d{3})(\\d{3})(\\d{4})', '\\1-\\2-\\3') %>% str_c(collapse = ', ')))

  # get detail
  customers_data <- pblapply(account_ids, safely(gads_customer), verbose = FALSE)

  # get res and errors
  res <- transpose(customers_data)

  # check errors
  if ( length(res$error) > 0 ) {

    for ( err in res$error ) {

      if ( is.null(err) ) next

      cli_alert_danger(err$message)

    }

  }

  # bind
  res <- tibble(data = list.filter(res$result, length(.) > 1)) %>%
         unnest_wider('data') %>%
         select(!where(is.list)) %>%
         rename_with(getOption('gads.column.name.case.fun'))

  # success msg
  cli_alert_success('Success! Loaded {nrow(res)} rows!')

  # return data
  return(res)
}

#' Get Keyword Dictionary From Google Ads Client Account
#'
#' @inheritParams gads_get_report
#' @param fields character vector, list of report fields, all report has own fields list, for example \href{https://developers.google.com/google-ads/api/fields/v23/keyword_view_query_builder}{see field list of keyword report}.
#'
#' @return tibble with Keyword criterions dicrionary
#' @export
#'
#' @seealso \href{https://developers.google.com/google-ads/api/fields/v23/keyword_view_query_builder}{Google Ads Query Builder}
gads_get_keywords <- function(
    customer_id = getOption('gads.customer.id'),
    fields      = c('ad_group_criterion.criterion_id',
                    'ad_group_criterion.keyword.text',
                    'ad_group_criterion.keyword.match_type',
                    'ad_group_criterion.status',
                    'ad_group_criterion.approval_status',
                    'ad_group_criterion.system_serving_status',
                    'ad_group_criterion.quality_info.quality_score',
                    'ad_group_criterion.quality_info.creative_quality_score',
                    'ad_group_criterion.quality_info.post_click_quality_score',
                    'ad_group.id',
                    'ad_group.name',
                    'ad_group.status',
                    'campaign.id',
                    'campaign.name',
                    'customer.id',
                    'customer.descriptive_name',
                    'metrics.average_cpc',
                    'metrics.average_cost',
                    'metrics.ctr',
                    'metrics.bounce_rate'),
    where                 = NULL,
    order_by              = NULL,
    limit                 = NULL,
    parameters            = NULL,
    login_customer_id     = getOption('gads.login.customer.id'),
    include_resource_name = FALSE,
    cl                    = NULL,
    verbose               = TRUE
) {

  # user gads_get_report
  res <- gads_get_report(
    resource          = 'keyword_view',
    fields            = fields,
    where             = where,
    order_by          = order_by,
    limit             = limit,
    parameters        = parameters,
    customer_id       = customer_id,
    login_customer_id = login_customer_id,
    verbose           = verbose,
    date_from         = NULL,
    date_to           = NULL,
    cl                = cl
  )

  # renaming to snake case
  res <- rename_with(res, function(x) str_remove( str_to_lower(x), 'ad_?group_?criterion\\_?'), matches('ad\\_?group\\_?criterion\\_?', ignore.case = TRUE) ) %>%
         rename_with(getOption('gads.column.name.case.fun'))

  return(res)

}

#' function for fix names in get_report
#'
#' @param x character, column names
#'
#' @return new columns names
#'
gads_fix_names <- function(x) {
  out <- str_remove(str_to_lower(x), 'metrics\\_|segments\\_')
  return(out)
}


gads_fix_names_regexp <- function(x, regexp = 'metrics\\_|segments\\_') {
  out <- str_remove(str_to_lower(x), regexp)
  return(out)
}

#' Get Ad Group Criterions Dictionary From Google Ads Client Account
#'
#' @inheritParams gads_get_report
#'
#' @param fields character vector, list of report fields, all report has own fields list, for example \href{https://developers.google.com/google-ads/api/fields/v23/ad_group}{see field list of ad group report}.
#'
#' @seealso \href{https://developers.google.com/google-ads/api/fields/v23/ad_group_criterion_query_builder}{Google Ads Query Builder}
#'
#' @return tibble with ad group criterions dicrionary
#' @export
#' @examples
#' \dontrun{
#' # set client customer id
#' gads_set_login_customer_id('xxx-xxx-xxxx')
#'
#' # set manager id if you work under MCC
#' gads_set_customer_id('xxx-xxx-xxxx')
#'
#' # load ad groups keywords list
#' kw <- gads_get_ad_group_criterions()
#'
#' }
gads_get_ad_group_criterions <- function(
  customer_id           = getOption('gads.customer.id'),
  fields                = c('ad_group_criterion.ad_group',
                            'ad_group_criterion.age_range.type',
                            'ad_group_criterion.app_payment_model.type',
                            'ad_group_criterion.approval_status',
                            'ad_group_criterion.bid_modifier',
                            'ad_group_criterion.combined_audience.combined_audience',
                            'ad_group_criterion.cpc_bid_micros',
                            'ad_group_criterion.cpm_bid_micros',
                            'ad_group_criterion.cpv_bid_micros',
                            'ad_group.id',
                            'customer.id',
                            'customer.descriptive_name',
                            'ad_group_criterion.criterion_id',
                            'ad_group_criterion.custom_affinity.custom_affinity',
                            'ad_group_criterion.custom_audience.custom_audience',
                            'ad_group_criterion.custom_intent.custom_intent',
                            'ad_group_criterion.disapproval_reasons',
                            'ad_group_criterion.display_name',
                            'ad_group_criterion.effective_cpc_bid_micros',
                            'ad_group_criterion.effective_cpc_bid_source',
                            'ad_group_criterion.effective_cpm_bid_micros',
                            'ad_group_criterion.effective_cpm_bid_source',
                            'ad_group_criterion.effective_cpv_bid_micros',
                            'ad_group_criterion.effective_cpv_bid_source',
                            'ad_group_criterion.effective_percent_cpc_bid_micros',
                            'ad_group_criterion.effective_percent_cpc_bid_source',
                            'ad_group_criterion.final_mobile_urls',
                            'ad_group_criterion.final_url_suffix',
                            'ad_group_criterion.final_urls',
                            'ad_group_criterion.gender.type',
                            'ad_group_criterion.income_range.type',
                            'ad_group_criterion.keyword.match_type',
                            'ad_group_criterion.keyword.text',
                            'ad_group_criterion.labels',
                            'ad_group_criterion.listing_group.case_value.hotel_city.city_criterion',
                            'ad_group_criterion.listing_group.case_value.hotel_class.value',
                            'ad_group_criterion.listing_group.case_value.hotel_id.value',
                            'ad_group_criterion.listing_group.case_value.hotel_state.state_criterion',
                            'ad_group_criterion.listing_group.case_value.product_brand.value',
                            'ad_group_criterion.listing_group.case_value.product_channel.channel',
                            'ad_group_criterion.listing_group.case_value.product_condition.condition',
                            'ad_group_criterion.listing_group.case_value.product_custom_attribute.index',
                            'ad_group_criterion.listing_group.case_value.product_custom_attribute.value',
                            'ad_group_criterion.listing_group.case_value.product_type.level',
                            'ad_group_criterion.listing_group.case_value.product_item_id.value',
                            'ad_group_criterion.listing_group.case_value.product_type.value',
                            'ad_group_criterion.listing_group.parent_ad_group_criterion',
                            'ad_group_criterion.listing_group.type',
                            'ad_group_criterion.mobile_app_category.mobile_app_category_constant',
                            'ad_group_criterion.mobile_application.app_id',
                            'ad_group_criterion.mobile_application.name',
                            'ad_group_criterion.negative',
                            'ad_group_criterion.parental_status.type',
                            'ad_group_criterion.percent_cpc_bid_micros',
                            'ad_group_criterion.placement.url',
                            'ad_group_criterion.position_estimates.estimated_add_cost_at_first_position_cpc',
                            'ad_group_criterion.position_estimates.estimated_add_clicks_at_first_position_cpc',
                            'ad_group_criterion.position_estimates.first_page_cpc_micros',
                            'ad_group_criterion.position_estimates.first_position_cpc_micros',
                            'ad_group_criterion.position_estimates.top_of_page_cpc_micros',
                            'ad_group_criterion.quality_info.creative_quality_score',
                            'ad_group_criterion.quality_info.post_click_quality_score',
                            'ad_group_criterion.quality_info.quality_score',
                            'ad_group_criterion.quality_info.search_predicted_ctr',
                            'ad_group_criterion.resource_name',
                            'ad_group_criterion.status',
                            'ad_group_criterion.system_serving_status',
                            'ad_group_criterion.topic.path',
                            'ad_group_criterion.topic.topic_constant',
                            'ad_group_criterion.tracking_url_template',
                            'ad_group_criterion.type',
                            'ad_group_criterion.url_custom_parameters',
                            'ad_group_criterion.user_interest.user_interest_category',
                            'ad_group_criterion.user_list.user_list',
                            'ad_group_criterion.webpage.conditions',
                            'ad_group_criterion.webpage.coverage_percentage',
                            'ad_group_criterion.webpage.criterion_name',
                            'ad_group_criterion.webpage.sample.sample_urls',
                            'ad_group_criterion.youtube_channel.channel_id',
                            'ad_group_criterion.youtube_video.video_id'),
  where                 = NULL,
  order_by              = NULL,
  limit                 = NULL,
  parameters            = NULL,
  login_customer_id     = getOption('gads.login.customer.id'),
  include_resource_name = FALSE,
  cl                    = NULL,
  verbose               = TRUE
) {

  # user gads_get_report
  res <- gads_get_report(
    resource          = 'ad_group_criterion',
    fields            = fields,
    where             = where,
    order_by          = order_by,
    limit             = limit,
    parameters        = parameters,
    customer_id       = customer_id,
    login_customer_id = login_customer_id,
    verbose           = verbose,
    date_from         = NULL,
    date_to           = NULL,
    cl                = cl
  )

  # renaming to snale case
  res <- rename_with(res, function(x) str_remove( str_to_lower(x), 'ad_?group_?criterion\\_?'), matches('ad\\_?group\\_?criterion\\_?', ignore.case = TRUE) ) %>%
    rename_with(getOption('gads.column.name.case.fun'))

  return(res)

}

#' Returns the requested Keyword Plan historical metrics.
#'
#' @param keyword_plan_id Keyword plan id, you can get list of your keyword plans using \code{\link{gads_get_report}} with recource keyword_plan
#' @inheritParams gads_get_report
#'
#' @return tibble with keyword plan historical metrics
#' @export
#'
#' @examples
#' \dontrun{
#' # set client id
#' gads_set_customer_id('xxx-xxx-xxxx')
#'
#' # set manager id
#' gads_set_login_customer_id('xxx-xxx-xxxx')
#'
#' # get list of plan
#' plan_data <- gads_get_report(
#'   resource = 'keyword_plan',
#'   fields = c('keyword_plan.id')
#' )
#'
#' # get keyword historical data
#' historical_plan_data <- gads_keyword_plan_historical_metrics(
#'  keyword_plan_id = plan_data$keyword_plan_id[1]#'
#' )
#'
#' # main plan data
#' data <- historical_plan_data$main_data
#' historical_data <- historical_plan_data$historical_data
#'
#' }
gads_keyword_plan_historical_metrics <- function(
  keyword_plan_id,
  customer_id       = getOption('gads.customer.id'),
  login_customer_id = getOption('gads.login.customer.id'),
  verbose           = TRUE
) {

  # delete - in customer id
  customer_id <- str_replace_all(customer_id, '-', '')

  # to env
  gads_customer_id_to_env(customer_id)

  # info
  if (verbose) cli_alert_info(c('Loading data: ', str_replace(customer_id, '(\\d{3})(\\d{3})(\\d{4})', '\\1-\\2-\\3')))

  # info
  if (verbose) cli_alert_info('Send query')

  # build query
  out <- request_build(
    method   = "POST",
    path     = str_glue('{options("gads.api.version")}/customers/{customer_id}/keywordPlans/{keyword_plan_id}:generateHistoricalMetrics'),
    token    = gads_token(),
    base_url = getOption('gads.base.url')
  )

  # send request
  ans <- request_retry(
    out,
    encode = "multipart",
    add_headers(`developer-token`= gads_developer_token(),
                `login-customer-id` = login_customer_id)
  )

  # request id
  rq_ids <- headers(ans)$`request-id`
  rgoogleads$last_request_id <- rq_ids

  # info
  if (verbose) cli_alert_info('Get response')

  # pars result
  data <- response_process(ans, error_message = gads_check_errors2)

  # info
  if (verbose) cli_alert_info('Parsing result')

  # main data
  res <- tibble(data = data$metrics) %>%
    unnest_wider('data') %>%
    unnest_wider('keywordMetrics') %>%
    select(-'monthlySearchVolumes') %>%
    relocate('searchQuery', .before = everything()) %>%
    rename_with(getOption('gads.column.name.case.fun'))

  # fix cost
  if ( any(  str_detect(str_to_lower(names(res) ), 'micros')) ) {

    if (verbose) cli_alert_info('Fix cost fields')

    res <- mutate(res,
                  across(matches('micros', ignore.case = TRUE), function(x) round(as.numeric(x) / 1000000, 2 )) ) %>%
      rename_with(gads_fix_names_regexp, matches('micros', ignore.case = TRUE), regexp = "\\_micros|micros")

  }

  # historical data
  historical <- tibble(data = data$metrics) %>%
    unnest_wider('data') %>%
    unnest_wider('keywordMetrics') %>%
    select('searchQuery', 'monthlySearchVolumes') %>%
    unnest_longer('monthlySearchVolumes') %>%
    unnest_wider('monthlySearchVolumes')%>%
    rename_with(getOption('gads.column.name.case.fun'))

  # success msg
  if (verbose) cli_alert_success('Success! Loaded {nrow(res)} rows!')
  if (verbose) cli_alert_info('For get data use res$main_data and res$historical_data')

  # to list
  res <- list(main_data = res, historical_data = historical)

  # return
  return(res)

}

---
title: "Миграция с RAdwords на rgoogleads"
author: "Алексей Селезнёв"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Миграция с RAdwords на rgoogleads}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  eval = FALSE,
  comment = "#>"
)
```

Пакет `RAdwords` работает с Google AdWords API версии 201809. Данный API давно не обновляется и прекратит работу 27 апреля 2022 года.

В этой виньетке мы подробно разберёмся с тем, как перейти с `RAdwords` на `rgoogleads`, для того, чтобы с апреля 2022 года ваши скрипты по прежнему корректно собирали необходимые данные из рекламных аккаунтов в Google Ads.

# Возможности пакета rgoogleads

* авторизация в API Google Ads;
* загрузка списка аккаунтов верхнего уровня;
* загрузка всей иерархии аккаунтов из управляющих аккаунтов;
* загрузка объектов рекламного кабинета: кампании, группы объявлений, объявления и другое;
* загрузка статистических данных из рекламных аккаунтов;
* загрузка метаданных ресурсов, полей ресурсов, сегментов и метрик;
* загрузка прогноза и исторических данных из планировщика ключевых слов.

# Преимущества пакета rgoogleads

Сначала давайте разберёмся, какие преимущества дает переход на новый пакет `rgoogleads`:

* `rgoogleads` работает с Google Ads API v8 (релиз от 09.06.2021), `RAdwords` работает с Google AdWords API v201809. Google AdWords  API прекратит работу 27.04.2022;
* для авторизации rgoogleads использует пакет gargle, что даёт гораздо больше гибкости по сравнению с тем,  как устроен процесс авторизации в RAdwords;
* в `rgoogleads` есть вшитый токен разработчика Google Ads и OAuth клиент для авторизации. Это избавит большинство пользователей от необходимости запрашивать у поддержки Google базовый доступ к API Google Ads и тратить время на создание проекта и OAuth клиента в Google Cloud Console;
* у большинства функций rgoogleads аргумент cl, который позволяет осуществлять загрузку данных в многопоточном режиме;
* в отличие от `RAdwords`, у `rgoogleads` есть функции для загрузки списков и иерархии аккаунтов;
* у `rgoogleads` есть отдельные функции для загрузки основных объектов рекламных кабинетов, таких как рекламные кампании, группы объявлений, ключевые слова и объявления;
* за счёт того, что запрос данных не разделён на отдельные функции, синтаксис rgoogleads более понятный и лаконичный. В RAdwords вам необходимо было изначально создать запрос функцией statement(), после чего использовать его для запроса данных в функции getData();
* у `rgoogleads` нет проблем при загрузке названий, содержащих кириллицу;
* если запрос к API столкнулся со сбоем на сервере (статус ответа 429 или выше), пакет `rgoogleads` автоматически выждет паузу в 100 секунд и повторит попытку запросить данные. За счёт этого работа данного пакета более стабильна и устойчива к сбоям на сервере Google Ads API;
* `rgoogleads` выводит подробное сообщение об ошибке. Для сравнения, если пользователь допустил ошибку в составлении запроса, `RAdwords` не выводит никакие сообщения;
* `rgoogleads` позволяет запрашивать данные из планировщика ключевых слов.
* используя `rgoogleads` вы можете запрашивать статистику по объектам, которые появились в Google Ads после сентября 2018 года.

# Основные различия между Google AdWords API и Google AdsAPI

К счастью, ключевых различий между устаревшим и новым API не так много, и процесс миграции не должен вызвать у вас особых сложностей. Ниже я перечислю ключевые моменты миграции.

* нет необходимости менять автоионизационные данные, токен разработчика, id и secret OAuth клиента будут работать также и с новым Google Ads API;
* в AdWords API отчёты были выделены в отдельную службу, в Google Ads API отчёты являются частью одного сервиса. Вам достаточно включить в отчёты необходимые поля с метриками, которые нужно получить;
* В AdWords API существовали типы отчётов, например, CAMPAINGN_PERFORMANCE_REPORT. В Google Ads API их нет, вместо типов отчётов представлен огромный набор ресурсов;
* у AdWords API и Google Ads API разный формат ответа от API;
* в Google Ads API отсутствует параметр includeZeroImpressions, вместо него вы можете использовать фильтр metrics.impressions > 0.

# Миграция с RAdwords на rgoogleads

```{r, eval = TRUE, results = "asis", echo=FALSE}

migrate_table <- data.frame(
  operation = c("Авторизация", "Запрос метаданных", "Запрос отчётов"),
  radwords = c("doAuth()", "reports(), metrics()", "statement() + getData()"),
  rgoogleads = c("gads_auth_configure() + gads_auth()", "gads_get_metadata(), gads_get_fields()", "gads_get_report()")
  )

DT::datatable(
  migrate_table, 
  colnames = c("Операция", "RAdwords", "rgoogleads"),
  options = list(pageLength = 5, dom = 'tip'))


```

Бывшие в Google AdWords типы отчётов, в Google Ads стали ресурсами. Ниже — таблица сопоставления из официальной справки:


```{r, eval = TRUE, results = "asis", echo=FALSE}
library(dplyr)
library(rvest)
reports_table <- data.frame(
  adwords = c("ACCOUNT_PERFORMANCE_REPORT", "AD_PERFORMANCE_REPORT", "ADGROUP_PERFORMANCE_REPORT", "AGE_RANGE_PERFORMANCE_REPORT",
              "AUDIENCE_PERFORMANCE_REPORT", "AUTOMATIC_PLACEMENTS_PERFORMANCE_REPORT", "BID_GOAL_PERFORMANCE_REPORT", "BUDGET_PERFORMANCE_REPORT",
              "CALL_METRICS_CALL_DETAILS_REPORT", "CAMPAIGN_AD_SCHEDULE_TARGET_REPORT", "CAMPAIGN_CRITERIA_REPORT", "CAMPAIGN_PERFORMANCE_REPORT",
              "CAMPAIGN_SHARED_SET_REPORT", "CAMPAIGN_LOCATION_TARGET_REPORT", "CLICK_PERFORMANCE_REPORT", "DISPLAY_KEYWORD_PERFORMANCE_REPORT"),
  ads     = c("customer", "ad_group_ad", "ad_group", "age_range_view",
              "campaign_audience_view, ad_group_audience_view", "group_placement_view", "bidding_strategy", "campaign_budget",
              "call_view", "ad_schedule_view", "campaign_criterion", "campaign",
              "campaign_shared_set", "location_view", "click_view", "display_keyword_view"))
  
# reports <- read_html("https://developers.google.com/google-ads/api/docs/migration/mapping") %>% 
#            html_element(css = ".responsive") %>% 
#            html_table(header = TRUE)

DT::datatable(
  reports_table, 
  colnames = c("Тип отчёта в Google AdWords API", "Ресурс в Google Ads API"),
  options = list(pageLength = 20)
  )

```

Соответствие полей «Отчёт» и полей ресурсов можно найти в официальной справке. Таблица очень большая, поэтому не вижу смысла дублировать её здесь.

Ниже приведу пример запроса отчёта по эффективности рекламных кампаний с одним и тем же набором полей, с помощью пакета `RAdwords` и `rgoogleads`.

## Запрос отчёта по эффективности рекламных кампаний с помощью RAdwords

```{r}
library(RAdwords)

# авторизация
adwords_auth <- doAuth()

# составляем запрос
query <- statement(
  select = c('CampaignName',
            'Date',
            'Clicks'),
  report = 'CAMPAIGN_PERFORMANCE_REPORT',
  start  = '2021-06-01',
  end    = '2021-06-30'
)

# загрузка данных
data1 <- getData(
  clientCustomerId = 'xxx-xxx-xxxx',
  statement        = query, 
  google_auth      = adwords_auth
)

```


## Запрос отчёта по эффективности рекламных кампаний с помощью rgoogleads

```{r}
library(rgoogleads)

# авторизация
gads_auth_configure(path = 'D:/ga_auth/app.json')
gads_auth(email = 'me@gmail.com')

# загрузка данных
data2 <- gads_get_report(
  resource = 'campaign',
  fields   = c('campaign.name',
              'segments.date',
              'metrics.clicks'),
  date_from         = '2021-06-01',
  date_to           = '2021-06-30',
  customer_id       = 'xxx-xxx-xxxx',
  login_customer_id = 'xxx-xxx-xxxx'
)
```

---
title: "Migrate from RAdwords to rgoogleads"
output: rmarkdown::html_vignette
author: "Alexey Seleznev"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{Migrate from RAdwords to rgoogleads}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  eval = FALSE,
  comment = "#>"
)
```

The problem is that the `RAdwords` package is used to work with Google AdWords API v201809. This API has not been updated for a long time and will stop working on 27 April 2022. 

In this vignette, I want to share some information on a new package, rgoogleads, which I started working on in June 2021. The package currently has all the functions needed to request the function data. Next, we will go through the details of how to switch from RAdwords to rgoogleads, so that from April 2022, your scripts will still correctly collect the necessary data from your Google Ads accounts.

# rgoogleads package features

The rgoogleads package currently includes all the functions needed to fetch data from the Google Ads API:

* authorize in Google Ads API;
* import the list of top-level accounts;
* import the entire account hierarchy from management accounts;
* import the ad account objects: campaigns, ad groups, ads, and more;
* import statistical data from advertising accounts;
* import resource metadata, resource fields, segments, and metrics;
* import forecast and historical data from the keyword planner.

# rgoogleads package benefits

Now let's look at the benefits of switching to the new rgoogleads package:

* `rgoogleads` is used to work with Google Ads API v8 (released on 09.06.2021); `RAdwords` is used to work with Google AdWords API v201809. Google AdWords API will sunset on 27.04.2022;
* `rgoogleads` uses the gargle package for authorization, which gives much more flexibility than the `RAdwords` authorization process;
* `rgoogleads` has an embedded Google Ads developer token and an OAuth client for authorization. This will save most users from having to request basic access to the Google Ads API from Google support and wasting time creating a project and OAuth client in the Google Cloud Console;
* most `rgoogleads` functions have a cl argument, which allows for multi-threaded data import;
* unlike `RAdwords`, `rgoogleads` has a list and account hierarchy import function;
* `rgoogleads` has separate functions to import the main objects of the advertising accounts, such as advertising campaigns, ad groups, keywords, and ads;
* since data request is not divided into separate functions, the syntax of `rgoogleads` is more straightforward and concise. In `RAdwords`, you had to initially create `statement()` function and then use it to request the data in the `getData()` function;
* `rgoogleads` has no problem importing names containing Cyrillic characters;
* if the API request encounters a server failure (response status 429 or higher), the `rgoogleads` package will automatically pause for 100 seconds and try to request data again. This makes this package more stable and resilient to Google Ads API server failures;
* `rgoogleads` displays a detailed error message. In comparison, `RAdwords` will not display a message if the user has made a request error;
* `rgoogleads` allows you to request data from the Keyword Planner.

# Key differences between the Google AdWords API and Google Ads API

Fortunately, there are few key differences between the old and new APIs, and the migration process isn’t that difficult. Let me enumerate the key points of the migration below.

* no need to change auto-ionization data; the developer token, ID, and client secret OAuth can also be used for a new Google Ads API;
* The AdWords API reporting is a separate service, while Google Ads API reporting is part of the same service. All you need to do is include the necessary metrics fields in your reports;
* In AdWords API, there were report types such as CAMPAINGN_PERFORMANCE_REPORT. The Google Ads API doesn't have them; instead of report types, it consists of a vast number of resources;
* AdWords API and Google Ads API have different API response formats;
* there is no includeZeroImpressions parameter in Google Ads API; instead, you can use the metrics.impressions > 0 filter.

# Migrating from `RAdwords` to `rgoogleads`

```{r, eval = TRUE, results = "asis", echo=FALSE}

migrate_table <- data.frame(
  operation = c("Authorization", "Metadata request", "Report request"),
  radwords = c("doAuth()", "reports(), metrics()", "statement() + getData()"),
  rgoogleads = c("gads_auth_configure() + gads_auth()", "gads_get_metadata(), gads_get_fields()", "gads_get_report()")
  )

DT::datatable(
  migrate_table, 
  colnames = c("Operation", "RAdwords", "rgoogleads"),
  options = list(pageLength = 5, dom = 'tip'))


```

The former report types in Google AdWords have become resources in Google Ads. See the comparison table from the official help guide:


```{r, eval = TRUE, results = "asis", echo=FALSE}
library(dplyr)
library(rvest)
reports_table <- data.frame(
  adwords = c("ACCOUNT_PERFORMANCE_REPORT", "AD_PERFORMANCE_REPORT", "ADGROUP_PERFORMANCE_REPORT", "AGE_RANGE_PERFORMANCE_REPORT",
              "AUDIENCE_PERFORMANCE_REPORT", "AUTOMATIC_PLACEMENTS_PERFORMANCE_REPORT", "BID_GOAL_PERFORMANCE_REPORT", "BUDGET_PERFORMANCE_REPORT",
              "CALL_METRICS_CALL_DETAILS_REPORT", "CAMPAIGN_AD_SCHEDULE_TARGET_REPORT", "CAMPAIGN_CRITERIA_REPORT", "CAMPAIGN_PERFORMANCE_REPORT",
              "CAMPAIGN_SHARED_SET_REPORT", "CAMPAIGN_LOCATION_TARGET_REPORT", "CLICK_PERFORMANCE_REPORT", "DISPLAY_KEYWORD_PERFORMANCE_REPORT"),
  ads     = c("customer", "ad_group_ad", "ad_group", "age_range_view",
              "campaign_audience_view, ad_group_audience_view", "group_placement_view", "bidding_strategy", "campaign_budget",
              "call_view", "ad_schedule_view", "campaign_criterion", "campaign",
              "campaign_shared_set", "location_view", "click_view", "display_keyword_view"))
  
# reports <- read_html("https://developers.google.com/google-ads/api/docs/migration/mapping") %>% 
#            html_element(css = ".responsive") %>% 
#            html_table(header = TRUE)

DT::datatable(
  reports_table, 
  colnames = c("Тип отчёта в Google AdWords API", "Ресурс в Google Ads API"),
  options = list(pageLength = 20)
  )

```

See the official help guide for the correspondence between the "Report" and the resource fields. The table is large, so there’s no need to duplicate it here.

See the example of requesting a campaign performance report with the same set of fields, using the `RAdwords` package and `rgoogleads` below.

## The request of ad campaign performance report using RAdwords

```{r}
library(RAdwords)

# auth
adwords_auth <- doAuth()

# create request
query <- statement(
  select = c('CampaignName',
            'Date',
            'Clicks'),
  report = 'CAMPAIGN_PERFORMANCE_REPORT',
  start  = '2021-06-01',
  end    = '2021-06-30'
)

# data import
data1 <- getData(
  clientCustomerId = 'xxx-xxx-xxxx',
  statement        = query, 
  google_auth      = adwords_auth
)

```


## The request of ad campaign performance report using rgoogleads

```{r}
library(rgoogleads)

# auth
gads_auth_configure(path = 'D:/ga_auth/app.json')
gads_auth(email = 'me@gmail.com')

# data import
data2 <- gads_get_report(
  resource = 'campaign',
  fields   = c('campaign.name',
              'segments.date',
              'metrics.clicks'),
  date_from         = '2021-06-01',
  date_to           = '2021-06-30',
  customer_id       = 'xxx-xxx-xxxx',
  login_customer_id = 'xxx-xxx-xxxx'
)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_get_ads.R
\name{gads_get_ads}
\alias{gads_get_ads}
\title{Get Ads Dictionary From Google Ads Client Account}
\usage{
gads_get_ads(
  fields = c("ad_group_ad.ad.id", "ad_group_ad.ad.name",
    "ad_group_ad.ad.added_by_google_ads", "ad_group_ad.ad.app_ad.descriptions",
    "ad_group_ad.ad.app_ad.headlines", "ad_group_ad.ad.app_ad.html5_media_bundles",
    "ad_group_ad.ad.app_ad.images", "ad_group_ad.ad.app_ad.mandatory_ad_text",
    "ad_group_ad.ad.app_engagement_ad.videos", "ad_group_ad.ad.device_preference",
    "ad_group_ad.ad.display_upload_ad.display_upload_product_type",
    "ad_group_ad.ad.display_upload_ad.media_bundle", "ad_group_ad.ad.display_url",
    "ad_group_ad.ad.expanded_dynamic_search_ad.description", 
    
    "ad_group_ad.ad.expanded_dynamic_search_ad.description2",
    "ad_group_ad.ad.expanded_text_ad.description",
    "ad_group_ad.ad.expanded_text_ad.description2",
    "ad_group_ad.ad.expanded_text_ad.headline_part1",
    "ad_group_ad.ad.expanded_text_ad.headline_part2",
    "ad_group_ad.ad.expanded_text_ad.headline_part3",
    "ad_group_ad.ad.expanded_text_ad.path1", "ad_group_ad.ad.expanded_text_ad.path2",
    "ad_group_ad.ad.final_url_suffix", "ad_group_ad.ad.final_urls",
    "ad_group_ad.ad.final_mobile_urls", "ad_group_ad.ad.hotel_ad", 
    
    "ad_group_ad.ad.image_ad.image_url", "ad_group_ad.ad.image_ad.mime_type",
    "ad_group_ad.ad.image_ad.name", "ad_group_ad.ad.image_ad.pixel_height",
    "ad_group_ad.ad.image_ad.pixel_width", "ad_group_ad.ad.image_ad.preview_image_url",
    "ad_group_ad.ad.image_ad.preview_pixel_height",
    "ad_group_ad.ad.image_ad.preview_pixel_width",
    "ad_group_ad.ad.legacy_app_install_ad",
    "ad_group_ad.ad.legacy_responsive_display_ad.accent_color",
    "ad_group_ad.ad.legacy_responsive_display_ad.allow_flexible_color",
    "ad_group_ad.ad.legacy_responsive_display_ad.business_name", 
    
    "ad_group_ad.ad.legacy_responsive_display_ad.description",
    "ad_group_ad.ad.legacy_responsive_display_ad.call_to_action_text",
    "ad_group_ad.ad.legacy_responsive_display_ad.format_setting",
    "ad_group_ad.ad.legacy_responsive_display_ad.logo_image",
    "ad_group_ad.ad.legacy_responsive_display_ad.long_headline",
    "ad_group_ad.ad.legacy_responsive_display_ad.main_color",
    "ad_group_ad.ad.legacy_responsive_display_ad.marketing_image",
    "ad_group_ad.ad.legacy_responsive_display_ad.price_prefix",
    "ad_group_ad.ad.legacy_responsive_display_ad.promo_text", 
    
    "ad_group_ad.ad.legacy_responsive_display_ad.short_headline",
    "ad_group_ad.ad.legacy_responsive_display_ad.square_logo_image",
    "ad_group_ad.ad.legacy_responsive_display_ad.square_marketing_image",
    "ad_group_ad.ad.local_ad.call_to_actions", "ad_group_ad.ad.local_ad.descriptions",
    "ad_group_ad.ad.local_ad.headlines", "ad_group_ad.ad.local_ad.logo_images",
    "ad_group_ad.ad.local_ad.marketing_images", "ad_group_ad.ad.local_ad.path1",
    "ad_group_ad.ad.local_ad.path2", "ad_group_ad.ad.resource_name", 
    
    "ad_group_ad.ad.responsive_display_ad.accent_color",
    "ad_group_ad.ad.responsive_display_ad.allow_flexible_color",
    "ad_group_ad.ad.responsive_display_ad.business_name",
    "ad_group_ad.ad.responsive_display_ad.call_to_action_text",
    "ad_group_ad.ad.responsive_display_ad.control_spec.enable_asset_enhancements",
    "ad_group_ad.ad.responsive_display_ad.control_spec.enable_autogen_video",
    "ad_group_ad.ad.responsive_display_ad.format_setting",
    "ad_group_ad.ad.responsive_display_ad.headlines",
    "ad_group_ad.ad.responsive_display_ad.long_headline", 
    
    "ad_group_ad.ad.responsive_display_ad.main_color",
    "ad_group_ad.ad.responsive_display_ad.price_prefix",
    "ad_group_ad.ad.responsive_display_ad.promo_text",
    "ad_group_ad.ad.responsive_display_ad.square_marketing_images",
    "customer.descriptive_name", "customer.id"),
  where = NULL,
  order_by = NULL,
  limit = NULL,
  parameters = NULL,
  customer_id = getOption("gads.customer.id"),
  login_customer_id = getOption("gads.login.customer.id"),
  include_resource_name = FALSE,
  cl = NULL,
  verbose = TRUE
)
}
\arguments{
\item{fields}{character vector, list pf report fields, all report has own fields list, for example \href{https://developers.google.com/google-ads/api/fields/v23/ad_group_ad}{see field list of ads report}.}

\item{where}{Filter, for example you can filter campaigns by status \code{where = "campaign.status = 'ENABLED'"}.}

\item{order_by}{Sorting, character vectors of fields and sorting directions, for example \code{order_by = c("campaign.name DESC", "metrics.clicks")}.}

\item{limit}{Maximun rows in report}

\item{parameters}{Query parameters, for example \code{parameters = "include_drafts=true"}.}

\item{customer_id}{Google Ads client customer id, supports a single account id: "xxx-xxx-xxxx" or a vector of ids from the same Google Ads MCC: c("xxx-xxx-xxxx", "xxx-xxx-xxxx")}

\item{login_customer_id}{Google Ads manager customer id}

\item{include_resource_name}{Get resource names fields in report}

\item{cl}{A cluster object created by \code{\link{makeCluster}}, or an integer to indicate number of child-processes (integer values are ignored on Windows) for parallel evaluations (see Details on performance).}

\item{verbose}{Console log output}
}
\value{
tibble with ads dicrionary
}
\description{
Get Ads Dictionary From Google Ads Client Account
}
\examples{
\dontrun{
# set client customer id
gads_set_login_customer_id('xxx-xxx-xxxx')

# set manager id if you work under MCC
gads_set_customer_id('xxx-xxx-xxxx')

# load ads list
myads <- gads_get_ads(
   fields = c("ad_group_ad.ad.id",
              "customer.descriptive_name"),
   where = 'ad_group_ad.status = "ENABLED"'
)

}
}
\seealso{
\href{https://developers.google.com/google-ads/api/fields/v23/ad_group_ad_query_builder}{Google Ads Query Builder}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_auth.R
\name{gads_has_token}
\alias{gads_has_token}
\title{Is there a token on hand?}
\usage{
gads_has_token()
}
\value{
Logical.
}
\description{
Reports whether rgoogleads has stored a token, ready for use in downstream
requests.
}
\seealso{
Other low-level API functions: 
\code{\link{gads_token}()}
}
\concept{low-level API functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_get_report.R
\name{gads_get_report}
\alias{gads_get_report}
\title{Get data from Google Ads API}
\usage{
gads_get_report(
  resource = "campaign",
  fields = c("campaign.id", "campaign.name", "customer.id", "customer.descriptive_name",
    "campaign.status", "segments.date", "metrics.all_conversions", "metrics.clicks",
    "metrics.cost_micros", "metrics.ctr", "metrics.impressions",
    "metrics.interaction_rate", "metrics.interactions", "metrics.invalid_clicks"),
  where = NULL,
  order_by = NULL,
  limit = NULL,
  parameters = NULL,
  date_from = Sys.Date() - 15,
  date_to = Sys.Date() - 1,
  during = c(NA, "TODAY", "YESTERDAY", "LAST_7_DAYS", "LAST_BUSINESS_WEEK", "THIS_MONTH",
    "LAST_MONTH", "LAST_14_DAYS", "LAST_30_DAYS", "THIS_WEEK_SUN_TODAY",
    "THIS_WEEK_MON_TODAY", "LAST_WEEK_SUN_SAT", "LAST_WEEK_MON_SUN"),
  customer_id = getOption("gads.customer.id"),
  login_customer_id = getOption("gads.login.customer.id"),
  include_resource_name = FALSE,
  gaql_query = NULL,
  cl = NULL,
  verbose = TRUE
)
}
\arguments{
\item{resource}{Report type, you can get list of all acessible resource using \code{\link{gads_get_metadata}}. For more information see \href{https://developers.google.com/google-ads/api/fields/v23/overview#list-of-all-resources}{link with list of all resources}}

\item{fields}{character vector, list of report fields, all report has own fields list. You can get list of accesible resource fields using \code{\link{gads_get_fields}} for example \href{https://developers.google.com/google-ads/api/fields/v23/campaign}{see field list of campaign report}.}

\item{where}{Filter, for example you can filter campaigns by status \code{where = "campaign.status = 'ENABLED'"}.}

\item{order_by}{Sorting, character vectors of fields and sorting directions, for example \code{order_by = c("campaign.name DESC", "metrics.clicks")}.}

\item{limit}{Maximun rows in report}

\item{parameters}{Query parameters, for example \code{parameters = "include_drafts=true"}.}

\item{date_from}{Beginning of date range. Format: 2018-01-01}

\item{date_to}{End of date rage. Format: 2018-01-10}

\item{during}{Predefined date range. See \href{https://developers.google.com/google-ads/api/docs/query/date-ranges#predefined_date_range}{documentation} for more details.}

\item{customer_id}{Google Ads client customer id, supports a single account id: "xxx-xxx-xxxx" or a vector of ids from the same Google Ads MCC: c("xxx-xxx-xxxx", "xxx-xxx-xxxx")}

\item{login_customer_id}{Google Ads manager customer id}

\item{include_resource_name}{Get resource names fields in report}

\item{gaql_query}{GAQL Query, you can make it in \code{\link{gads_get_metadata}}. For more information see \href{https://developers.google.com/google-ads/api/fields/v23/overview_query_builder}{Query Builder}. If you use gaql_query, you don't need set other query parameters like resource, fields, where, dates etc.}

\item{cl}{A cluster object created by \code{\link{makeCluster}}, or an integer to indicate number of child-processes (integer values are ignored on Windows) for parallel evaluations (see Details on performance).}

\item{verbose}{Console log output}
}
\value{
tibble with the Google Ads Data.
}
\description{
Get data from Google Ads API
}
\examples{
\dontrun{
# set client id
gads_set_login_customer_id('xxx-xxx-xxxx')

# set manager id if you work under MCC
gads_set_customer_id('xxx-xxx-xxxx')

# default paramas is campaign performance report
campaign_stat <- gads_get_report()


# you can load data from several client accounts at once
# from the same Google Ads MCC
# client ids
accounts <- c('xxx-xxx-xxxx', 'yyy-yyy-yyyy')
# loading data
multi_rep <- gads_get_report(
    date_from = as.Date('2021-06-10'),
    date_to = as.Date('2021-06-17'),
    customer_id = accounts
)

# ------------------
# using more arguments for other reports
group_report <- gads_get_report(
customer_id = 4732519773,
resource    = "ad_group",
fields = c("ad_group.campaign",
           "ad_group.id",
           "ad_group.name",
           "ad_group.status",
           "metrics.clicks",
           "metrics.cost_micros"),
date_from   = "2021-06-10",
date_to     = "2021-06-17",
where       = "ad_group.status = 'ENABLED'",
order_by    = c("metrics.clicks DESC", "metrics.cost_micros"),
limit       = 30000
)

# ------------------
# parallel loading mode
# note: you must using login_customer_id agrument in parallel mode
# because oprions gads_set_login_customer_id() does't work in parallel mode loading
library(parallel)

# make core cluster
cl <- makeCluster(4)

# loading data
multi_rep <- gads_get_report(
  date_from         = as.Date('2021-06-10'),
  date_to           = as.Date('2021-06-17'),
  customer_id       = c('111-111-1111',
                        '222-222-2222',
                        '333-333-3333',
                        '444-444-4444',
                        '555-555-5555'),
  login_customer_id = "999-999-9999",
  cl                = cl
)

# stop cluster
stopCluster(cl)
}
}
\seealso{
\itemize{
\item \href{https://developers.google.com/google-ads/api/fields/v23/overview}{Oficial Google Ads API Reports documantation}
\item \href{https://developers.google.com/google-ads/api/fields/v23/overview_query_builder}{Google Ads Query Builder}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_get_account_hierarchy.R
\name{gads_get_account_hierarchy}
\alias{gads_get_account_hierarchy}
\title{Get Google Ads Manager Account Hierarchy}
\usage{
gads_get_account_hierarchy(
  manager_customer_id = getOption("gads.login.customer.id"),
  include_drafts = FALSE,
  login_customer_id = getOption("gads.login.customer.id")
)
}
\arguments{
\item{manager_customer_id}{ID of the manager account whose hierarchy you want to get.}

\item{include_drafts}{logical, Incliding drafts child account.}

\item{login_customer_id}{Ypor top-level manager account id.}
}
\value{
tibble with data of all the child accounts
}
\description{
Get Google Ads Manager Account Hierarchy
}
\examples{
\dontrun{
acc_hier <- gads_get_account_hierarchy(
    manager_customer_id = '111-111-1111',
    login_customer_id   = '000-000-0000')
}
}
\seealso{
\href{https://developers.google.com/google-ads/api/docs/account-management/get-account-hierarchy}{Get Account Hierarchy API documentation}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_last_request_ids.R
\name{gads_last_request_ids}
\alias{gads_last_request_ids}
\title{Get last API request ID for Google Ads API support ticket}
\usage{
gads_last_request_ids()
}
\value{
Request ID
}
\description{
Get last API request ID for Google Ads API support ticket
}
\examples{
\dontrun{
gads_last_request_ids()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_auth.R
\name{gads_token}
\alias{gads_token}
\title{Produce configured token}
\usage{
gads_token()
}
\value{
A \code{request} object (an S3 class provided by \link[httr:httr-package]{httr}).
}
\description{
For internal use or for those programming around the Google Ads API.
Returns a token pre-processed with \code{\link[httr:config]{httr::config()}}. Most users
do not need to handle tokens "by hand" or, even if they need some
control, \code{\link[=gads_auth]{gads_auth()}} is what they need. If there is no current
token, \code{\link[=gads_auth]{gads_auth()}} is called to either load from cache or
initiate OAuth2.0 flow.
If auth has been deactivated via \code{\link[=gads_deauth]{gads_deauth()}}, \code{gads_token()}
returns \code{NULL}.
}
\seealso{
Other low-level API functions: 
\code{\link{gads_has_token}()}
}
\concept{low-level API functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_fix_names.R
\name{gads_fix_names}
\alias{gads_fix_names}
\title{function for fix names in get_report}
\usage{
gads_fix_names(x)
}
\arguments{
\item{x}{character, column names}
}
\value{
new columns names
}
\description{
function for fix names in get_report
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_auth.R
\name{gads_deauth}
\alias{gads_deauth}
\title{Suspend authorization}
\usage{
gads_deauth()
}
\value{
only suspend authorization
}
\description{
Put rgoogleads into a de-authorized state. Instead of sending a token,
rgoogleads will send an API key. This can be used to access public
resources for which no Google sign-in is required. This is handy for using
rgoogleads in a non-interactive setting to make requests that do not
require a token. It will prevent the attempt to obtain a token
interactively in the browser. The user can configure their own API key
via \code{\link[=gads_auth_configure]{gads_auth_configure()}} and retrieve that key via
\code{\link[=gads_api_key]{gads_api_key()}}.
In the absence of a user-configured key, a built-in default key is used.
}
\seealso{
Other auth functions: 
\code{\link{gads_auth}()},
\code{\link{gads_auth_configure}()}
}
\concept{auth functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_get_ad_groups.R
\name{gads_get_ad_groups}
\alias{gads_get_ad_groups}
\title{Get Ad Groups Dictionary From Google Ads Client Account}
\usage{
gads_get_ad_groups(
  customer_id = getOption("gads.customer.id"),
  fields = c("ad_group.id", "ad_group.name", "ad_group.status",
    "ad_group.ad_rotation_mode", "ad_group.base_ad_group", "ad_group.campaign",
    "campaign.id", "ad_group.display_custom_bid_dimension",
    "ad_group.effective_target_cpa_source", "ad_group.effective_target_roas",
    "ad_group.effective_target_roas_source", "ad_group.final_url_suffix",
    "ad_group.target_roas", "ad_group.type", "ad_group.url_custom_parameters",
    "ad_group.tracking_url_template", "customer.id", "customer.descriptive_name"),
  where = NULL,
  order_by = NULL,
  limit = NULL,
  parameters = NULL,
  login_customer_id = getOption("gads.login.customer.id"),
  include_resource_name = FALSE,
  cl = NULL,
  verbose = TRUE
)
}
\arguments{
\item{customer_id}{Google Ads client customer id, supports a single account id: "xxx-xxx-xxxx" or a vector of ids from the same Google Ads MCC: c("xxx-xxx-xxxx", "xxx-xxx-xxxx")}

\item{fields}{character vector, list of report fields, all report has own fields list, for example \href{https://developers.google.com/google-ads/api/fields/v23/ad_group}{see field list of ad group report}.}

\item{where}{Filter, for example you can filter campaigns by status \code{where = "campaign.status = 'ENABLED'"}.}

\item{order_by}{Sorting, character vectors of fields and sorting directions, for example \code{order_by = c("campaign.name DESC", "metrics.clicks")}.}

\item{limit}{Maximun rows in report}

\item{parameters}{Query parameters, for example \code{parameters = "include_drafts=true"}.}

\item{login_customer_id}{Google Ads manager customer id}

\item{include_resource_name}{Get resource names fields in report}

\item{cl}{A cluster object created by \code{\link{makeCluster}}, or an integer to indicate number of child-processes (integer values are ignored on Windows) for parallel evaluations (see Details on performance).}

\item{verbose}{Console log output}
}
\value{
tibble with ad group dicrionary
}
\description{
Get Ad Groups Dictionary From Google Ads Client Account
}
\examples{
\dontrun{
# set client customer id
gads_set_login_customer_id('xxx-xxx-xxxx')

# set manager id if you work under MCC
gads_set_customer_id('xxx-xxx-xxxx')

# load ad groups list
adgroups <- gads_get_ad_groups(
    where = 'ad_group.status = "ENABLED"'
)

}
}
\seealso{
\href{https://developers.google.com/google-ads/api/fields/v23/ad_group_query_builder}{Google Ads Query Builder}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_get_geo_targets.R
\name{gads_get_geo_targets}
\alias{gads_get_geo_targets}
\title{Download CSV of geo targets}
\usage{
gads_get_geo_targets(
  doc_page = "https://developers.google.com/google-ads/api/reference/data/geotargets",
  file_link = "auto"
)
}
\arguments{
\item{doc_page}{Link to Google Ads API Reference page}

\item{file_link}{Link to csv file, default is 'auto'}
}
\value{
data.frame with geo targets dictionary
}
\description{
Download CSV of geo targets
}
\examples{
\dontrun{
geo_dict <- gads_get_geo_targets()
}
}
\seealso{
\href{https://developers.google.com/google-ads/api/reference/data/geotargets?hl=en}{Google Ads Geo Targets document page}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_auth.R
\name{gads_auth}
\alias{gads_auth}
\title{Authorization in Google Ads API}
\usage{
gads_auth(
  email = gargle::gargle_oauth_email(),
  path = NULL,
  cache = gargle::gargle_oauth_cache(),
  use_oob = gargle::gargle_oob_default(),
  developer_token = getOption("gads.developer.token"),
  token = NULL
)
}
\arguments{
\item{email}{Optional. Allows user to target a specific Google identity.}

\item{path}{Path to JSON file with identifying the service account}

\item{cache}{Specifies the OAuth token cache.}

\item{use_oob}{Whether to prefer "out of band" authentication.}

\item{developer_token}{Your Google Ads Developer Token.}

\item{token}{A token with class \link[httr:Token-class]{Token2.0} or an object of}
}
\value{
\link[httr:Token-class]{Token2.0}
}
\description{
Authorize rgoogleads to view and manage your Google Ads Account. This function is a
wrapper around \code{\link[gargle:token_fetch]{gargle::token_fetch()}}.

By default, you are directed to a web browser, asked to sign in to your
Google account, and to grant rgoogleads permission to operate on your
behalf with Google Ads. By default, with your permission, these user
credentials are cached in a folder below your home directory, from where
they can be automatically refreshed, as necessary. Storage at the user
level means the same token can be used across multiple projects and
tokens are less likely to be synced to the cloud by accident.
}
\details{
Most users, most of the time, do not need to call \code{gads_auth()}
explicitly -- it is triggered by the first action that requires
authorization. Even when called, the default arguments often suffice.

However, when necessary, \code{gads_auth()} allows the user to explicitly:
\itemize{
\item Declare which Google identity to use, via an \code{email} specification.
\item Use a service account token or workload identity federation via
\code{path}.
\item Bring your own \code{token}.
\item Customize \code{scopes}.
\item Use a non-default \code{cache} folder or turn caching off.
\item Explicitly request out-of-band (OOB) auth via \code{use_oob}.
}

If you are interacting with R within a browser (applies to RStudio
Server, Posit Workbench, Posit Cloud, and Google Colaboratory), you need
OOB auth or the pseudo-OOB variant. If this does not happen
automatically, you can request it explicitly with \code{use_oob = TRUE} or,
more persistently, by setting an option via
\code{options(gargle_oob_default = TRUE)}.

The choice between conventional OOB or pseudo-OOB auth is determined
by the type of OAuth client. If the client is of the "installed" type,
\code{use_oob = TRUE} results in conventional OOB auth. If the client is of
the "web" type, \code{use_oob = TRUE} results in pseudo-OOB auth. Packages
that provide a built-in OAuth client can usually detect which type of
client to use. But if you need to set this explicitly, use the
\code{"gargle_oauth_client_type"} option:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{options(gargle_oauth_client_type = "web")       # pseudo-OOB
# or, alternatively
options(gargle_oauth_client_type = "installed") # conventional OOB
}\if{html}{\out{</div>}}

For details on the many ways to find a token, see
\code{\link[gargle:token_fetch]{gargle::token_fetch()}}. For deeper control over auth, use
\code{\link[=gads_auth_configure]{gads_auth_configure()}} to bring your own OAuth client or API key.
To learn more about gargle options, see \link[gargle:gargle_options]{gargle::gargle_options}.
}
\examples{
\dontrun{
## load/refresh existing credentials, if available
## otherwise, go to browser for authentication and authorization
gads_auth()

## force use of a token associated with a specific email
gads_auth(email = "yourname@example.com")

## force a menu where you can choose from existing tokens or
## choose to get a new one
gads_auth(email = NA)

## -----------------------
## use own developer token
gads_auth(
    email = "yourname@example.com",
    developer_token = "your developer token"
)

## -----------------------
## use own OAuth client app
gads_auth_configure(
    path = "path/to/your/oauth_client.json"
)

gads_auth(email = "yourname@example.com")
}
}
\seealso{
Other auth functions: 
\code{\link{gads_auth_configure}()},
\code{\link{gads_deauth}()}
}
\concept{auth functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_get_ad_group_criterions.R
\name{gads_get_ad_group_criterions}
\alias{gads_get_ad_group_criterions}
\title{Get Ad Group Criterions Dictionary From Google Ads Client Account}
\usage{
gads_get_ad_group_criterions(
  customer_id = getOption("gads.customer.id"),
  fields = c("ad_group_criterion.ad_group", "ad_group_criterion.age_range.type",
    "ad_group_criterion.app_payment_model.type", "ad_group_criterion.approval_status",
    "ad_group_criterion.bid_modifier",
    "ad_group_criterion.combined_audience.combined_audience",
    "ad_group_criterion.cpc_bid_micros", "ad_group_criterion.cpm_bid_micros",
    "ad_group_criterion.cpv_bid_micros", "ad_group.id", "customer.id",
    "customer.descriptive_name", "ad_group_criterion.criterion_id",
    "ad_group_criterion.custom_affinity.custom_affinity", 
    
    "ad_group_criterion.custom_audience.custom_audience",
    "ad_group_criterion.custom_intent.custom_intent",
    "ad_group_criterion.disapproval_reasons", "ad_group_criterion.display_name",
    "ad_group_criterion.effective_cpc_bid_micros",
    "ad_group_criterion.effective_cpc_bid_source",
    "ad_group_criterion.effective_cpm_bid_micros",
    "ad_group_criterion.effective_cpm_bid_source",
    "ad_group_criterion.effective_cpv_bid_micros",
    "ad_group_criterion.effective_cpv_bid_source",
    "ad_group_criterion.effective_percent_cpc_bid_micros", 
    
    "ad_group_criterion.effective_percent_cpc_bid_source",
    "ad_group_criterion.final_mobile_urls", "ad_group_criterion.final_url_suffix",
    "ad_group_criterion.final_urls", "ad_group_criterion.gender.type",
    "ad_group_criterion.income_range.type", "ad_group_criterion.keyword.match_type",
    "ad_group_criterion.keyword.text", "ad_group_criterion.labels",
    "ad_group_criterion.listing_group.case_value.hotel_city.city_criterion",
    "ad_group_criterion.listing_group.case_value.hotel_class.value",
    "ad_group_criterion.listing_group.case_value.hotel_id.value", 
    
    "ad_group_criterion.listing_group.case_value.hotel_state.state_criterion",
    "ad_group_criterion.listing_group.case_value.product_brand.value",
    "ad_group_criterion.listing_group.case_value.product_channel.channel",
    "ad_group_criterion.listing_group.case_value.product_condition.condition",
    "ad_group_criterion.listing_group.case_value.product_custom_attribute.index",
    "ad_group_criterion.listing_group.case_value.product_custom_attribute.value",
    "ad_group_criterion.listing_group.case_value.product_type.level", 
    
    "ad_group_criterion.listing_group.case_value.product_item_id.value",
    "ad_group_criterion.listing_group.case_value.product_type.value",
    "ad_group_criterion.listing_group.parent_ad_group_criterion",
    "ad_group_criterion.listing_group.type",
    "ad_group_criterion.mobile_app_category.mobile_app_category_constant",
    "ad_group_criterion.mobile_application.app_id",
    "ad_group_criterion.mobile_application.name", "ad_group_criterion.negative",
    "ad_group_criterion.parental_status.type",
    "ad_group_criterion.percent_cpc_bid_micros", 
    
    "ad_group_criterion.placement.url",
    "ad_group_criterion.position_estimates.estimated_add_cost_at_first_position_cpc",
    "ad_group_criterion.position_estimates.estimated_add_clicks_at_first_position_cpc",
    "ad_group_criterion.position_estimates.first_page_cpc_micros",
    "ad_group_criterion.position_estimates.first_position_cpc_micros",
    "ad_group_criterion.position_estimates.top_of_page_cpc_micros",
    "ad_group_criterion.quality_info.creative_quality_score",
    "ad_group_criterion.quality_info.post_click_quality_score", 
    
    "ad_group_criterion.quality_info.quality_score",
    "ad_group_criterion.quality_info.search_predicted_ctr",
    "ad_group_criterion.resource_name", "ad_group_criterion.status",
    "ad_group_criterion.system_serving_status", "ad_group_criterion.topic.path",
    "ad_group_criterion.topic.topic_constant",
    "ad_group_criterion.tracking_url_template", "ad_group_criterion.type",
    "ad_group_criterion.url_custom_parameters",
    "ad_group_criterion.user_interest.user_interest_category",
    "ad_group_criterion.user_list.user_list", 
    
    "ad_group_criterion.webpage.conditions",
    "ad_group_criterion.webpage.coverage_percentage",
    "ad_group_criterion.webpage.criterion_name",
    "ad_group_criterion.webpage.sample.sample_urls",
    "ad_group_criterion.youtube_channel.channel_id",
    "ad_group_criterion.youtube_video.video_id"),
  where = NULL,
  order_by = NULL,
  limit = NULL,
  parameters = NULL,
  login_customer_id = getOption("gads.login.customer.id"),
  include_resource_name = FALSE,
  cl = NULL,
  verbose = TRUE
)
}
\arguments{
\item{customer_id}{Google Ads client customer id, supports a single account id: "xxx-xxx-xxxx" or a vector of ids from the same Google Ads MCC: c("xxx-xxx-xxxx", "xxx-xxx-xxxx")}

\item{fields}{character vector, list of report fields, all report has own fields list, for example \href{https://developers.google.com/google-ads/api/fields/v23/ad_group}{see field list of ad group report}.}

\item{where}{Filter, for example you can filter campaigns by status \code{where = "campaign.status = 'ENABLED'"}.}

\item{order_by}{Sorting, character vectors of fields and sorting directions, for example \code{order_by = c("campaign.name DESC", "metrics.clicks")}.}

\item{limit}{Maximun rows in report}

\item{parameters}{Query parameters, for example \code{parameters = "include_drafts=true"}.}

\item{login_customer_id}{Google Ads manager customer id}

\item{include_resource_name}{Get resource names fields in report}

\item{cl}{A cluster object created by \code{\link{makeCluster}}, or an integer to indicate number of child-processes (integer values are ignored on Windows) for parallel evaluations (see Details on performance).}

\item{verbose}{Console log output}
}
\value{
tibble with ad group criterions dicrionary
}
\description{
Get Ad Group Criterions Dictionary From Google Ads Client Account
}
\examples{
\dontrun{
# set client customer id
gads_set_login_customer_id('xxx-xxx-xxxx')

# set manager id if you work under MCC
gads_set_customer_id('xxx-xxx-xxxx')

# load ad groups keywords list
kw <- gads_get_ad_group_criterions()

}
}
\seealso{
\href{https://developers.google.com/google-ads/api/fields/v23/ad_group_criterion_query_builder}{Google Ads Query Builder}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_auth.R
\name{gads_user}
\alias{gads_user}
\title{Get info on current user}
\usage{
gads_user()
}
\value{
An email address or, if no token has been loaded, \code{NULL}.
}
\description{
Reveals the email address of the user associated with the current token.
If no token has been loaded yet, this function does not initiate auth.
}
\seealso{
\code{\link[gargle:token-info]{gargle::token_userinfo()}}, \code{\link[gargle:token-info]{gargle::token_email()}},
\code{\link[gargle:token-info]{gargle::token_tokeninfo()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_last_request_ids.R
\name{gads_customer_id_to_env}
\alias{gads_customer_id_to_env}
\title{Write customer id for error message}
\usage{
gads_customer_id_to_env(customer_id)
}
\arguments{
\item{customer_id}{Your client customer id}
}
\value{
only set customer id into env
}
\description{
Write customer id for error message
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_last_request_ids.R
\name{gads_customer_id_from_env}
\alias{gads_customer_id_from_env}
\title{Get customer id for error message}
\usage{
gads_customer_id_from_env()
}
\value{
only set customer id into env
}
\description{
Get customer id for error message
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_check_errors.R
\name{gads_check_errors}
\alias{gads_check_errors}
\title{Helper function for check api answer on error}
\usage{
gads_check_errors(out, client_id = NULL, verbose = FALSE, request_id)
}
\arguments{
\item{out}{API answer}

\item{client_id}{Google Ads Customer id}

\item{verbose}{Console output}

\item{request_id}{Api request id}
}
\value{
stop the function when api request faild
}
\description{
Helper function for check api answer on error
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_get_accessible_customers.R
\name{gads_get_accessible_customers}
\alias{gads_get_accessible_customers}
\title{Get all data of customers directly accessible by the user authenticating the call.}
\usage{
gads_get_accessible_customers()
}
\value{
List of your accessible accounts from top level
}
\description{
Get all data of customers directly accessible by the user authenticating the call.
}
\examples{
\dontrun{
accounts <- gads_get_accessible_customers()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgoogleads_package.R
\docType{package}
\name{rgoogleads-package}
\alias{rgoogleads-package}
\alias{rgoogleads}
\title{Loading Data From 'Google Ads API'}
\description{
\if{html}{\figure{rgoogleads.png}{options: style='float: right' alt='logo' width='120'}}
Interface for loading data from 'Google Ads API',
see \url{https://developers.google.com/google-ads/api/docs/start}.
Package provide function for authorization and loading reports.

Capabilities of \code{rgoogleads}:
\itemize{
\item Authorization in the Google Ads API
\item Loading a list of top-level accounts
\item Loading the entire hierarchy of accounts from manager accounts
\item Loading list of Google Ads client account objects: campaigns, ad groups, ads, etc.
\item Loading statistics from Google Ads client account
\item Loading resource metadata, resource fields, segments and metrics
\item Loading forecast and historical metrics from Keyword Planning.
}
}
\examples{
\dontrun{
library(rgoogleads)

# set own oauth app
gads_auth_configure(path = 'C:/auth/app.json')
# set your developer token if needed, or use default developer token
gads_auth(email = 'me@gmail.com', developer_token = "own developer token")

# get list of accessible accounts
my_accounts <- gads_get_accessible_customers()

# set manager account id
gads_set_login_customer_id('xxx-xxx-xxxx')

# set client account id
gads_set_customer_id('xxx-xxx-xxxx')

# load report data
ad_group_report <- gads_get_report(
  resource    = "ad_group",
  fields = c("ad_group.campaign",
             "ad_group.id",
             "ad_group.name",
             "ad_group.status",
             "metrics.clicks",
             "metrics.cost_micros"),
  date_from   = "2021-06-10",
  date_to     = "2021-06-17",
  where       = "ad_group.status = 'ENABLED'",
  order_by    = c("metrics.clicks DESC", "metrics.cost_micros")
)
}

}
\seealso{
\itemize{
\item \href{https://www.youtube.com/playlist?list=PLD2LDq8edf4qprTxRcflDwV9IvStiChHi}{YouTube lessons playlist}
\item \href{https://developers.google.com/google-ads/api/docs/start}{Oficial Google Ads API documantation}
\item \href{https://developers.google.com/google-ads/api/fields/v23/overview_query_builder}{Google Ads Query Builder}
\item \href{https://selesnow.github.io/rgoogleads/docs/}{rgoogleads home page}
}
}
\author{
Alexey Seleznev
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_keyword_plan_forecast_metrics.R
\name{gads_keyword_plan_forecast_metrics}
\alias{gads_keyword_plan_forecast_metrics}
\title{Returns the requested Keyword Plan forecasts.}
\usage{
gads_keyword_plan_forecast_metrics(
  keyword_plan_id,
  customer_id = getOption("gads.customer.id"),
  login_customer_id = getOption("gads.login.customer.id"),
  verbose = TRUE
)
}
\arguments{
\item{keyword_plan_id}{Keyword plan id, you can get list of your keyword plans using \code{\link{gads_get_report}} with recource keyword_plan}

\item{customer_id}{Google Ads client customer id, supports a single account id: "xxx-xxx-xxxx" or a vector of ids from the same Google Ads MCC: c("xxx-xxx-xxxx", "xxx-xxx-xxxx")}

\item{login_customer_id}{Google Ads manager customer id}

\item{verbose}{Console log output}
}
\value{
tibble with keyword plan historical metrics
}
\description{
Returns the requested Keyword Plan forecasts.
}
\examples{
\dontrun{
# set client id
gads_set_customer_id('xxx-xxx-xxxx')

# set manager id
gads_set_login_customer_id('xxx-xxx-xxxx')

# get list of plan
plan_data <- gads_get_report(
  resource = 'keyword_plan',
  fields = c('keyword_plan.id')
)

# get keyword historical data
historical_plan_data <- gads_keyword_plan_forecast_metrics(
 keyword_plan_id = plan_data$keyword_plan_id[1]#'
)

}
}
\seealso{
\href{https://developers.google.com/google-ads/api/docs/keyword-planning/overview?hl=en}{Keyword Planning API Documentation}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_get_fields.R
\name{gads_get_fields}
\alias{gads_get_fields}
\title{Get resource or field information.}
\usage{
gads_get_fields(object_name)
}
\arguments{
\item{object_name}{name of resource, resource's field, segmentation field or metric}
}
\value{
List of resource or field metadata
}
\description{
Get resource or field information.
}
\examples{
\dontrun{
ad_group_info <- gads_get_fields("ad_group")
}
}
\seealso{
\href{https://developers.google.com/google-ads/api/docs/concepts/field-service}{Resource Metadata API documentation}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_get_keywords.R
\name{gads_get_keywords}
\alias{gads_get_keywords}
\title{Get Keyword Dictionary From Google Ads Client Account}
\usage{
gads_get_keywords(
  customer_id = getOption("gads.customer.id"),
  fields = c("ad_group_criterion.criterion_id", "ad_group_criterion.keyword.text",
    "ad_group_criterion.keyword.match_type", "ad_group_criterion.status",
    "ad_group_criterion.approval_status", "ad_group_criterion.system_serving_status",
    "ad_group_criterion.quality_info.quality_score",
    "ad_group_criterion.quality_info.creative_quality_score",
    "ad_group_criterion.quality_info.post_click_quality_score", "ad_group.id",
    "ad_group.name", "ad_group.status", "campaign.id", "campaign.name", "customer.id",
    "customer.descriptive_name", 
     "metrics.average_cpc", "metrics.average_cost",
    "metrics.ctr", "metrics.bounce_rate"),
  where = NULL,
  order_by = NULL,
  limit = NULL,
  parameters = NULL,
  login_customer_id = getOption("gads.login.customer.id"),
  include_resource_name = FALSE,
  cl = NULL,
  verbose = TRUE
)
}
\arguments{
\item{customer_id}{Google Ads client customer id, supports a single account id: "xxx-xxx-xxxx" or a vector of ids from the same Google Ads MCC: c("xxx-xxx-xxxx", "xxx-xxx-xxxx")}

\item{fields}{character vector, list of report fields, all report has own fields list, for example \href{https://developers.google.com/google-ads/api/fields/v23/keyword_view_query_builder}{see field list of keyword report}.}

\item{where}{Filter, for example you can filter campaigns by status \code{where = "campaign.status = 'ENABLED'"}.}

\item{order_by}{Sorting, character vectors of fields and sorting directions, for example \code{order_by = c("campaign.name DESC", "metrics.clicks")}.}

\item{limit}{Maximun rows in report}

\item{parameters}{Query parameters, for example \code{parameters = "include_drafts=true"}.}

\item{login_customer_id}{Google Ads manager customer id}

\item{include_resource_name}{Get resource names fields in report}

\item{cl}{A cluster object created by \code{\link{makeCluster}}, or an integer to indicate number of child-processes (integer values are ignored on Windows) for parallel evaluations (see Details on performance).}

\item{verbose}{Console log output}
}
\value{
tibble with Keyword criterions dicrionary
}
\description{
Get Keyword Dictionary From Google Ads Client Account
}
\seealso{
\href{https://developers.google.com/google-ads/api/fields/v23/keyword_view_query_builder}{Google Ads Query Builder}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_auth.R
\name{gads_auth_configure}
\alias{gads_auth_configure}
\alias{gads_auth_cache_path}
\alias{gads_open_auth_cache_folder}
\alias{gads_api_key}
\alias{gads_developer_token}
\alias{gads_oauth_app}
\title{Edit and view auth configuration}
\usage{
gads_auth_configure(
  client,
  path,
  api_key,
  developer_token,
  app = lifecycle::deprecated()
)

gads_auth_cache_path()

gads_open_auth_cache_folder()

gads_api_key()

gads_developer_token()

gads_oauth_app()
}
\arguments{
\item{client}{A Google OAuth client, presumably constructed via
\code{\link[gargle:gargle_oauth_client_from_json]{gargle::gargle_oauth_client_from_json()}}. Note, however, that it is
preferred to specify the client with JSON, using the \code{path} argument.}

\item{path}{JSON downloaded from \href{https://console.cloud.google.com}{Google Cloud Console}, containing a client id and
secret, in one of the forms supported for the \code{txt} argument of
\code{\link[jsonlite:fromJSON]{jsonlite::fromJSON()}} (typically, a file path or JSON string).}

\item{api_key}{API key.}

\item{developer_token}{Your Google Ads Developer Token.}

\item{app}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Replaced by the \code{client}
argument.}
}
\value{
\itemize{
\item \code{gads_auth_configure()}: An object of R6 class
\link[gargle:AuthState-class]{gargle::AuthState}, invisibly.
\item \code{gads_oauth_client()}: the current user-configured OAuth client.
\item \code{gads_api_key()}: the current user-configured API key.
}
}
\description{
These functions give more control over and visibility into the auth
configuration than \code{\link[=gads_auth]{gads_auth()}} does. \code{gads_auth_configure()}
lets the user specify their own:
\itemize{
\item OAuth client, which is used when obtaining a user token.
\item API key. If rgoogleads is de-authorized via \code{\link[=gads_deauth]{gads_deauth()}}, all
requests are sent with an API key in lieu of a token.
}

See the \code{vignette("get-api-credentials", package = "gargle")}
for more.
If the user does not configure these settings, internal defaults
are used.

\code{gads_oauth_client()} and \code{gads_api_key()} retrieve the
currently configured OAuth client and API key, respectively.
}
\examples{
\dontrun{
# see and store the current user-configured OAuth app (probaby `NULL`)
(original_app <- gads_oauth_app())

# see and store the current user-configured API key (probaby `NULL`)
(original_api_key <- gads_api_key())

if (require(httr)) {
  # bring your own app via client id (aka key) and secret
  google_app <- httr::oauth_app(
    "my-awesome-google-api-wrapping-package",
    key = "YOUR_CLIENT_ID_GOES_HERE",
    secret = "YOUR_SECRET_GOES_HERE"
  )
  google_key <- "YOUR_API_KEY"
  gads_auth_configure(app = google_app, api_key = google_key)

  # confirm the changes
  gads_oauth_app()
  gads_api_key()

  # bring your own app via JSON downloaded from Google Developers Console
  # this file has the same structure as the JSON from Google
  gads_auth_configure(path = app_path)

  # confirm the changes
  gads_oauth_app()

  # use own developer token
  gads_auth_configure(developer_token = 'Your developer token')

}

# restore original auth config
gs4_auth_configure(app = original_app, api_key = original_api_key)
}
}
\seealso{
Other auth functions: 
\code{\link{gads_auth}()},
\code{\link{gads_deauth}()}
}
\concept{auth functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_get_campaigns.R
\name{gads_get_campaigns}
\alias{gads_get_campaigns}
\title{Get Campaigns Dictionary From Google Ads Client Account}
\usage{
gads_get_campaigns(
  fields = c("campaign.id", "campaign.name", "campaign.accessible_bidding_strategy",
    "campaign.ad_serving_optimization_status", "campaign.advertising_channel_sub_type",
    "campaign.advertising_channel_type", "campaign.app_campaign_setting.app_id",
    "campaign.app_campaign_setting.app_store", "campaign.base_campaign",
    "campaign.bidding_strategy",
    "campaign.app_campaign_setting.bidding_strategy_goal_type",
    "campaign.campaign_budget", "campaign.bidding_strategy_type",
    "campaign.dynamic_search_ads_setting.language_code", 
    
    "campaign.start_date_time", "campaign.end_date_time", "campaign.status",
    "campaign.manual_cpm", "campaign.manual_cpv",
    "campaign.maximize_conversion_value.target_roas",
    "campaign.maximize_conversions.target_cpa_micros",
    "campaign.network_settings.target_content_network",
    "campaign.network_settings.target_google_search",
    "campaign.network_settings.target_partner_search_network",
    "campaign.network_settings.target_search_network",
    "campaign.optimization_goal_setting.optimization_goal_types",
    "campaign.optimization_score", 
     "campaign.payment_mode",
    "campaign.serving_status", "campaign.shopping_setting.campaign_priority",
    "campaign.target_roas.target_roas", "campaign.tracking_url_template",
    "customer.descriptive_name", "customer.id"),
  where = NULL,
  order_by = NULL,
  limit = NULL,
  parameters = NULL,
  customer_id = getOption("gads.customer.id"),
  login_customer_id = getOption("gads.login.customer.id"),
  include_resource_name = FALSE,
  cl = NULL,
  verbose = TRUE
)
}
\arguments{
\item{fields}{character vector, list of report fields, all report has own fields list, for example \href{https://developers.google.com/google-ads/api/fields/v23/campaign}{see field list of campaigns report}.}

\item{where}{Filter, for example you can filter campaigns by status \code{where = "campaign.status = 'ENABLED'"}.}

\item{order_by}{Sorting, character vectors of fields and sorting directions, for example \code{order_by = c("campaign.name DESC", "metrics.clicks")}.}

\item{limit}{Maximun rows in report}

\item{parameters}{Query parameters, for example \code{parameters = "include_drafts=true"}.}

\item{customer_id}{Google Ads client customer id, supports a single account id: "xxx-xxx-xxxx" or a vector of ids from the same Google Ads MCC: c("xxx-xxx-xxxx", "xxx-xxx-xxxx")}

\item{login_customer_id}{Google Ads manager customer id}

\item{include_resource_name}{Get resource names fields in report}

\item{cl}{A cluster object created by \code{\link{makeCluster}}, or an integer to indicate number of child-processes (integer values are ignored on Windows) for parallel evaluations (see Details on performance).}

\item{verbose}{Console log output}
}
\value{
tibble with campaings dicrionary
}
\description{
Get Campaigns Dictionary From Google Ads Client Account
}
\examples{
\dontrun{
# set client customer id
gads_set_login_customer_id('xxx-xxx-xxxx')

# set manager id if you work under MCC
gads_set_customer_id('xxx-xxx-xxxx')

# load campaing list
camps <- gads_get_campaigns(
    where = "campaign.status = 'ENABLED'"
)

}
}
\seealso{
\href{https://developers.google.com/google-ads/api/fields/v23/campaign_query_builder}{Google Ads Query Builder}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_customer.R
\name{gads_customer}
\alias{gads_customer}
\title{Get all information about Google Ads Customer}
\usage{
gads_customer(customer_id = getOption("gads.customer.id"), verbose = TRUE)
}
\arguments{
\item{customer_id}{Google Ads customer id}

\item{verbose}{Processing log output into console}
}
\value{
Google Ads customer data
}
\description{
Get all information about Google Ads Customer
}
\seealso{
\href{https://developers.google.com/google-ads/api/rest/common/search}{Method: SearchStream documentation}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_auth.R
\name{gads_set_customer_id}
\alias{gads_set_customer_id}
\title{Set client customer id in current R session}
\usage{
gads_set_customer_id(customer_id)
}
\arguments{
\item{customer_id}{your client customer id}
}
\value{
only set options
}
\description{
Set client customer id in current R session
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gads_get_metadata.R
\name{gads_get_metadata}
\alias{gads_get_metadata}
\title{Get metada of object, RESOURCE, ATTRIBUTE, METRIC or SEGMENT}
\usage{
gads_get_metadata(
  category = c("RESOURCE", "ATTRIBUTE", "METRIC", "SEGMENT", "ALL"),
  fields = c("name", "category", "data_type", "selectable", "filterable", "sortable",
    "selectable_with", "metrics", "segments", "is_repeated", "type_url", "enum_values",
    "attribute_resources")
)
}
\arguments{
\item{category}{Object category}

\item{fields}{Metadata fields}
}
\value{
tibble with object metadata
important arrays in result:
\describe{
\item{attributeResources}{Resources that can be using in \code{resource} argument in \code{\link{gads_get_report}}.}
\item{metrics}{Metrics that are available to be selected with the resource in the \code{field} argument in \code{\link{gads_get_report}}. Only populated for fields where the category is RESOURCE.}
\item{segments}{Segment keys that can be selected with the resource in the \code{field} argument in \code{\link{gads_get_report}}. These segment the metrics specified in the query. Only populated for fields where the category is RESOURCE.}
\item{selectableWith}{Fields that can be selected alongside a given field, when not in the FROM clause. This attribute is only relevant when identifying resources or segments that are able to be selected in a query where they are not included by the resource in the FROM clause. As an example, if we are selecting \code{ad_group.id} and \code{segments.date} from \code{ad_group}, and we want to include attributes from \code{campaign}, we would need to check that \code{segments.date} is in the selectableWith attribute for campaign, since it's being selected alongside the existing \code{segments.date} field.}
}
}
\description{
Get metada of object, RESOURCE, ATTRIBUTE, METRIC or SEGMENT
}
\examples{
\dontrun{
# get resource list
resources <- gads_get_metadata("RESOURCE")

# get list of all objects
metadata <- gads_get_metadata("ALL")

}
}
\seealso{
\href{https://ads-developers.googleblog.com/2021/04/the-query-builder-blog-series-part-3.html}{The Query Builder Blog Series: Part 3 - Creating a Resource Schema}
and \href{https://developers.google.com/google-ads/api/docs/concepts/field-service}{Resource Metadata API documentation}
}
